  //////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
//
// ScriptFunctionWaitForNextEvent.cpp
//
//	ScriptFunctionWaitForNextEvent class
//
///////////////////////////////////////
  ///////////////////////////////////////
  
/*

Copyright (c) 2005, Tim Burrell
All rights reserved.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#include "ScriptFunctionWaitForNextEvent.hpp"
#include "../libGizmo/Debug.hpp"
#include "../libGizmo/UtilTime.hpp"

////////////////////////////////////////////////////////////////////////////
// Statics / Type Defs
///////////////////////////////////////

////////////////////////////////////////////////////////////////////////////
// Overrides
///////////////////////////////////////

bool ScriptFunctionWaitForNextEvent::executeFunction() {
	m_pDaemon->onScriptInitialized();
	
	ScriptFuncArg * pRetArg = getOutArg(0);
	if (!pRetArg) {
		Debug::out("ScriptFunctionWaitForNextEvent :: executeFunction -- No Out Parameter!");
		return false;
	}
	
	m_EventSignalled = m_pDaemon->signalPluginsToCheckForEvents();
	if (m_EventSignalled) {
		pRetArg->ArgBoolean = true;
		return true;
	}
	UtilTime::nanoSleep(1);
	
	pRetArg->ArgBoolean = false;
	return true;
}

////////////////////////////////////////////////////////////////////////////
// Global Funcs
///////////////////////////////////////

////////////////////////////////////////////////////////////////////////////
// Construction
///////////////////////////////////////

ScriptFunctionWaitForNextEvent::ScriptFunctionWaitForNextEvent(GizmoDaemonInterface * pDaemon) {
	m_pDaemon = pDaemon;
	m_EventSignalled = false;
	
	FunctionName = "waitForNextEvent";
	FunctionDescription = "Waits for the next daemon event";
		
	addOutArg(SCRIPT_ARG_BOOLEAN, false);
}

ScriptFunctionWaitForNextEvent::~ScriptFunctionWaitForNextEvent() {
}

////////////////////////////////////////////////////////////////////////////
// Class Body
///////////////////////////////////////

void ScriptFunctionWaitForNextEvent::signalEvent() {
	m_EventSignalled = true;
}
