  //////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
//
// Debug.cpp
//
//	Debug class
//
///////////////////////////////////////
  ///////////////////////////////////////
  
/*

Copyright (c) 2005, Tim Burrell
All rights reserved.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#include <stdio.h>

#include "Debug.hpp"

////////////////////////////////////////////////////////////////////////////
// Constants
///////////////////////////////////////

const int DEBUG_BUFFER_SIZE = 1024;

////////////////////////////////////////////////////////////////////////////
// Statics
///////////////////////////////////////

bool Debug::m_DebugMode = false;
bool Debug::m_DisableAllOutput = false;
char * Debug::m_LogFile = "Debug.log";
String Debug::m_LastOutput;
void (* Debug::m_HookFunc) (const EDebugOutputType, const char *) = NULL;

////////////////////////////////////////////////////////////////////////////
// Construction
///////////////////////////////////////

Debug::Debug() {
}

Debug::~Debug() {
}

////////////////////////////////////////////////////////////////////////////
// Class Body
///////////////////////////////////////

void Debug::setDebugMode(bool DebugMode) {
	m_DebugMode = DebugMode;

	if (m_DebugMode)
		out("Debug :: DebugMode [enabled] ** BufferSize = [%d]\n", DEBUG_BUFFER_SIZE);
}

void Debug::setDisableAllOutput(bool Disable) {
	m_DisableAllOutput = Disable;
}

bool Debug::getDebugMode() {
	return m_DebugMode;
}

bool Debug::out(const char * TextToOutput, ...) {
	char		text[DEBUG_BUFFER_SIZE];
	va_list		ap;

	va_start(ap, TextToOutput);
		vsprintf(text, TextToOutput, ap);
	va_end(ap);

	return Debug::output(text, OUTPUT_NORMAL, false, false, false);
}

bool Debug::outDebug(const char * TextToOutput, ...) {
	if (!m_DebugMode)
		return true;
		
	char		text[DEBUG_BUFFER_SIZE];
	va_list		ap;

	va_start(ap, TextToOutput);
		vsprintf(text, TextToOutput, ap);
	va_end(ap);

	return Debug::output(text, OUTPUT_DEBUG, false, false, false);
}

bool Debug::outError(const char * TextToOutput, ...) {
	char		text[DEBUG_BUFFER_SIZE];
	va_list		ap;

	va_start(ap, TextToOutput);
		vsprintf(text, TextToOutput, ap);
	va_end(ap);

	return Debug::output(text, OUTPUT_ERROR, false, false, false);
}

bool Debug::outFatal(const char * TextToOutput, ...) {
	char		text[DEBUG_BUFFER_SIZE];
	va_list		ap;

	va_start(ap, TextToOutput);
		vsprintf(text, TextToOutput, ap);
	va_end(ap);

	return Debug::output(text, OUTPUT_FATAL, false, false, false);
}

bool Debug::outInfo(const char * TextToOutput, ...) {
	char		text[DEBUG_BUFFER_SIZE];
	va_list		ap;

	va_start(ap, TextToOutput);
		vsprintf(text, TextToOutput, ap);
	va_end(ap);

	return Debug::output(text, OUTPUT_INFO, false, false, false);
}

bool Debug::outWarn(const char * TextToOutput, ...) {
	char		text[DEBUG_BUFFER_SIZE];
	va_list		ap;

	va_start(ap, TextToOutput);
		vsprintf(text, TextToOutput, ap);
	va_end(ap);

	return Debug::output(text, OUTPUT_WARN, false, false, false);
}

bool Debug::output(const char * TextToOutput, EDebugOutputType Type, bool OutputToLog, bool DateStamp, bool ForceOutput, ...) {
	bool OutToScreen = false;
	bool ErrMode = false;

	if (m_DisableAllOutput)
		return true;

	char		text[DEBUG_BUFFER_SIZE];
	va_list		ap;

	va_start(ap, ForceOutput);
	    vsprintf(text, TextToOutput, ap);
	va_end(ap);

	switch (Type)
	{
	case OUTPUT_NORMAL:
		OutToScreen = true;
		break;

	case OUTPUT_INFO:
		OutToScreen = true;
		break;

	case OUTPUT_WARN:
		OutToScreen = true;
		break;

	case OUTPUT_ERROR:
		ErrMode = true;
		OutToScreen = true;
		break;

	case OUTPUT_FATAL:
		OutToScreen = true;
		break;

	case OUTPUT_DEBUG:
		OutToScreen = m_DebugMode;
		break;
	}

	if (m_HookFunc) {
		m_HookFunc(Type, text);
	} else if (OutToScreen) {
		m_LastOutput = text;
		printf("%s\n", text);
	} else if (ErrMode) {
		perror(text);
	}

	return true;
}

void Debug::setDebugMessageHook(void HookFunc(const EDebugOutputType Type, const char * Text)) {
	m_HookFunc = HookFunc;
}

String Debug::getLastOutput() {
	return m_LastOutput;
}

