////////////////////////////////////////////////////////////////////////////
//
// Directory.cpp
//
//	Directory main file
//
///////////////////////////////////////

/*

Copyright (c) 2005, Tim Burrell
All rights reserved.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/
 
#include <stddef.h>
#include <stdio.h>
#include <sys/types.h>
#include <dirent.h>
#include <string.h>

#include "Directory.hpp"
#include "Debug.hpp"
#include "FileIO.hpp"

////////////////////////////
// kon / dekon
//////////////

Directory::Directory() {
}

Directory::Directory(const char * Path) {
	scanDir(Path);
}

Directory::~Directory() {
	m_FileList.deleteAll();
}

////////////////////////////
// func bodys
//////////////

bool Directory::scanDir(const char * Path) {
	Debug::outDebug("Directory :: Scanning Dir [%s]", Path);
	
	String sPath = FileIO::relativeToAbsolute(Path);

	m_FileList.deleteAll();
	
	DIR *dp;
	struct dirent *ep;

	dp = opendir(sPath);
	if (dp != NULL) {
		m_Path = Path;
		while ((ep = readdir(dp)) != 0) {
			// check for dot thangers
			if ( (strcmp(ep->d_name, ".") == 0) || (strcmp(ep->d_name, "..") == 0) )
				continue;
				
			DirEnt * pEnt = new DirEnt();
			pEnt->FileName = ep->d_name;
			pEnt->FullPath = Path + String("/") + pEnt->FileName;
			
			m_FileList.addTail(pEnt);
		}
		
		(void) closedir (dp);
	} else {
		Debug::outDebug("Directory :: FAILED to Scan Dir [%s]", sPath.getString());
		
		return false;
	}
		
	return true;
}

int Directory::getNumFiles() {
	return m_FileList.getSize();
}

String Directory::getFileName(int Index) {
	String Ret;

	if (!isIndexValid(Index))
		return Ret;
		
	DirEnt * pEnt = m_FileList.getDataFromIndex(Index);

	Ret.setValue(pEnt->FileName);
	
	return Ret;
}

String Directory::getFileNameFullPath(int Index) {
	String Ret;

	if (!isIndexValid(Index))
		return Ret;
		
	DirEnt * pEnt = m_FileList.getDataFromIndex(Index);

	Ret.setValue(pEnt->FullPath);
	
	return Ret;
}

bool Directory::isIndexValid(int Index) {
	if ( (Index > -1) && (Index < getNumFiles()) )
		return true;
		
	return false;
}

String Directory::getScanPath() {
	return m_Path;
}

