////////////////////////////////////////////////////////////////////////////
//
// DynamicLibrary.cpp
//
//	DynamicLibrary main file
//
///////////////////////////////////////

/*

Copyright (c) 2005, Tim Burrell
All rights reserved.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#include "DynamicLibrary.hpp"

#include "FileIO.hpp"
#include "Debug.hpp"

////////////////////////////////////////////////////////////////////////////
// Statics / Type Defs
///////////////////////////////////////

////////////////////////////////////////////////////////////////////////////
// Construction
///////////////////////////////////////

DynamicLibrary::DynamicLibrary() {
	initialize();
}

DynamicLibrary::DynamicLibrary(const char * Library) {
	initialize();
	openLibrary(Library);
}

DynamicLibrary::~DynamicLibrary() {
	if (m_CloseOnDestruct)
		closeLibrary();
}

////////////////////////////////////////////////////////////////////////////
// Overridables
///////////////////////////////////////

////////////////////////////////////////////////////////////////////////////
// Class Body
///////////////////////////////////////

void DynamicLibrary::initialize() {
	m_Lib = NULL;
	m_LibName = "";
	m_CloseOnDestruct = true;
}

bool DynamicLibrary::openLibrary(const char * Library, EDynamicLibraryOpenType Type, bool OpenGlobal) {
	if (m_Lib)
		closeLibrary();

	String tLib = FileIO::relativeToAbsolute(Library);

	int flags = Type;
	if (OpenGlobal)
		flags |= RTLD_GLOBAL;

	m_Lib = dlopen(tLib, flags);

	if (!m_Lib) {
		Debug::outDebug("Failed to Open Library [%s] : %s", Library, dlerror());
		return false;
	}

	m_LibName = tLib;

	return true;
}

void DynamicLibrary::closeLibrary() {
	if (!m_Lib)
		return;

	dlclose(m_Lib);
	initialize();
}

void * DynamicLibrary::getFunction(const char * FunctionName) {
	if (!m_Lib)
		return NULL;

	return dlsym(m_Lib, FunctionName);
}

const char * DynamicLibrary::getError() {
	const char * ret = dlerror();

	if (!ret)
		return "";
		
	return ret;
}

void DynamicLibrary::setCloseOnDestruct(bool CloseOnDestruct) {
	m_CloseOnDestruct = CloseOnDestruct;
}

void DynamicLibrary::setToLibrary(DynamicLibrary & LibToSetTo) {
	m_Lib = LibToSetTo.m_Lib;
	m_LibName = LibToSetTo.m_LibName;
	LibToSetTo.m_CloseOnDestruct = false;
}

const char * DynamicLibrary::getLibraryPath() {
	return m_LibName;
}

bool DynamicLibrary::isLoaded() {
	return m_Lib != NULL;
}

