////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//
// DynamicLibrary.hpp
//
//	enEd header file
//
///////////////////////////////////////
  ///////////////////////////////////////
  
/*

Copyright (c) 2005, Tim Burrell
All rights reserved.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#ifndef __DynamicLibrary_h
#define __DynamicLibrary_h

#if HAVE_CONFIG_H
#include "config.h"
#endif

#include "String.hpp"

#include <dlfcn.h>
#include <stdlib.h>

//////////////////////////////////////////////////////////////////////////////
// Type Defs
///////////////////////////////////////

enum EDynamicLibraryOpenType {
	DYNLIB_OPEN_NOW = RTLD_NOW,
	DYNLIB_OPEN_LAZY = RTLD_LAZY
};

//////////////////////////////////////////////////////////////////////////////
// Class Definition
///////////////////////////////////////

class DynamicLibrary {
public:
	void 		closeLibrary();
	const char * 	getError();
	void *		getFunction(const char * FunctionName);
	const char * 	getLibraryPath();
	bool		isLoaded();
	bool		openLibrary(const char * Library, EDynamicLibraryOpenType = DYNLIB_OPEN_LAZY, bool OpenGlobal = true);
	void		setCloseOnDestruct(bool CloseOnDestruct);
	void 		setToLibrary(DynamicLibrary & LibToSetTo);
	
	DynamicLibrary();
	DynamicLibrary(const char * Library);
	virtual ~DynamicLibrary();

private:
	void		initialize();
	
	String		m_LibName;
	void *		m_Lib;
	bool		m_CloseOnDestruct;
};

#endif // __DynamicLibrary_h

