////////////////////////////////////////////////////////////////////////////
//
// FileIO.cpp
//
//	FileIO main file
//
///////////////////////////////////////

/*

Copyright (c) 2001-2005, Tim Burrell
All rights reserved.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#include <sys/stat.h>
#include <sys/types.h>
#include <stdio.h>
#include <stddef.h>
#include <stdlib.h>
#ifndef WIN32
#include <dirent.h>
#include <unistd.h>
#else
#include "../foundation/String.hpp"
#endif
#include <fcntl.h>

#include "FileIO.hpp"

FileStat::FileStat() {
	FileSize = -1;
}

FileStat::~FileStat() {
}

bool FileIO::doesFileExist(const char * FilePath) {
	FileStat Checker = getFileStats(FilePath);
	
	if (Checker.FileSize == -1)
		return false;
	else
		return true;
}

String FileIO::getCurrentDirectory() {
#ifdef WIN32
	TCHAR Buf[MAX_PATH + 1];
	String ret;
	
	DWORD Ret = ::GetCurrentDirectory(MAX_PATH, Buf);
	
	if (Ret == 0)
		return "";
	else
		ret = Buf;
	
	return ret;
#else
	return "";
#endif
}

bool FileIO::setCurrentDirectory(char * Directory)
{
#ifdef WIN32
	BOOL ret = ::SetCurrentDirectory(Directory);
	
	if (!ret)
		return false;
	
	return true;
#else
	return false;
#endif
}

bool FileIO::doesDirExist(const char * FilePath) {
#ifdef WIN32
	// make sure directory is valid
	String CurDir = FileIO::getCurrentDirectory();
	String Directory = FilePath;
	
	if (Directory.mid(Directory.getLength() - 1) == "\\")
		Directory = Directory.mid(0, Directory.getLength() - 1);
	
	if (!setCurrentDirectory((char *)Directory.getString()))
		return false;
	else
		setCurrentDirectory((char *)CurDir.getString());
	
	return true;
#else
	DIR *dp;
	//struct dirent *ep;

	dp = opendir(FilePath);
	if (dp != NULL) {
		(void) closedir (dp);
		return true;
	} else
		return false;
		
	return true;
#endif
}

FileStat FileIO::getFileStats(const char * FilePath) {
	FileStat Ret;
	
	struct stat stat_p;
	
	if (stat(FilePath, &stat_p) == -1)
		return Ret;
	else
		Ret.FileSize = stat_p.st_size;
	
	return Ret;
}

// NOT RECURSIVE YET!
bool FileIO::touchRecursive(const char * FilePath, bool DoRecursive) {
	FILE * t = fopen(FilePath, "a");
	if (!t)
		return false;

	fclose(t);
	return true;
}

String FileIO::relativeToAbsolute(String FilePath) {
	const char * home = getenv("HOME");

	if (home) {
		String sHome = home;
		sHome += "/";
		
		FilePath.replaceSubstring("~/", sHome);
	}

	return FilePath;
}

