  ///////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////
//
// GizmoDeviceInput.hpp
//
//	GizmoDeviceInput header file
//
///////////////////////////////////////
  ///////////////////////////////////////
  
/*

Copyright (c) 2005, Tim Burrell
All rights reserved.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#ifndef __GizmoDeviceInput_h
#define __GizmoDeviceInput_h

#if HAVE_CONFIG_H
#include "config.h"
#endif

#include "EventWatcherInterface.hpp"
#include "String.hpp"

//////////////////////////////////////////////////////////////////////////////
// Typedefs
///////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
// Class Definition
///////////////////////////////////////

class GizmoDeviceInput : public EventWatcherInterface {
public:
	virtual bool	checkEventsWaiting();
	int 		getKeyCodeFromString(String inString);
	const char * 	getKeyStringFromCode(int Code);
	bool 		grabExclusiveAccess(bool Grab);
	virtual void	onInputAbsolute(int Code, int Value);
	virtual void	onInputKey(int Code, int Value);
	virtual void	onInputLED(int Code, int Value);
	virtual void	onInputMisc(int Code, int Value);
	virtual void	onInputRelative(int Code, int Value);
	virtual void	onInputSyn(int Code, int Value);
	bool 		remapKey(int CurCode, int NewCode);
	bool 		writeToDevice(int Type, int Code, int Value);

	GizmoDeviceInput(const char * DevicePath);
	virtual ~GizmoDeviceInput();
	
protected:
	void 		closeDevice();
	bool		initDevice();
	bool 		openDevice(const char * DevName, int DeviceNumber = -1);

	String		m_DevicePath;
	String		m_DeviceName;
	int		m_FD;
	int		m_DevNum;
	
private:
	void 		processEvent(struct input_event * ev);

	static int	m_DevTotal;
};

#endif // __GizmoDeviceInput_h

