  ///////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////
//
// PThread.hpp
//
//	PThread header file
//
///////////////////////////////////////
  ///////////////////////////////////////
  
/*

Copyright (c) 2005, Tim Burrell
All rights reserved.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#ifndef __PThread_h
#define __PThread_h

#if HAVE_CONFIG_H
#include "config.h"
#endif

#include <pthread.h>
#include "String.hpp"

//////////////////////////////////////////////////////////////////////////////
// Class Definition
///////////////////////////////////////

class PThread {
public:
	bool		createThread();
	bool 		exitThread(void * ReturnVal);
	void *		getPThreadArguments();
	const char * 	getPThreadClassName();
	bool		hasExitSignalled();
	bool		isThreading();
	void		setPThreadCallback(void * (* ThreadFuncCallback) (PThread * TheThread));
	void		setPThreadExitCallback(void (* ThreadExitFuncCallback) (void));
	void		setPThreadName(const char * Name);
	
	virtual void	onThreadExiting();
	virtual void	onThreadExit(void * ReturnVal);
	virtual void 	* threadFunction();

	static	bool	waitOnThreadToExit(PThread * pPThread, long MaxWait = 0);
	        bool	waitOnToExit(long MaxWait = 0);

	friend void 	* PTHREAD_ThreadFunction(void * Arguments);

	static void 	setStabilized(bool Stable);
	static bool 	getStabilized();
	
	PThread(void * Arguments);
	virtual ~PThread();

protected:
	bool		m_ExitSignalled;
	
private:
	void 		reset();
	
	pthread_t 	m_ThreadID;
	void 		* m_Arguments;
	bool		m_Threading;
	String		m_ClassName;
	bool		m_Waiting;

	void * (* m_ThreadFuncCallback) (PThread * TheThread);
	void (* m_ThreadExitFuncCallback) (void);

	static bool	m_Stabilized;
};

#endif // __PThread_h

