  //////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
//
// String.cpp
//
//	String class
//
///////////////////////////////////////
  ///////////////////////////////////////
  
/*

Copyright (c) 2005, Tim Burrell
All rights reserved.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <stdio.h>
#include <stdarg.h>

#include "String.hpp"

#define	BUFFER_SIZE	1024

////////////////////////////////////////////////////////////////////////////
// Statics / Type Defs
///////////////////////////////////////

int String::m_FloatPrecision = 5;

////////////////////////////////////////////////////////////////////////////
// Construction
///////////////////////////////////////

String::String() {
	init();
}

String::String(const char * initVal) {
	init();
	setValue(initVal);
}

String::String(const String & initVal) {
	init();
	setValue(initVal);
}

String::String(const char initVal) {
	init();
	setValue(initVal);
}

String::~String() {
	freeString();
}

////////////////////////////////////////////////////////////////////////////
// Operator Overloads
///////////////////////////////////////

const String & String::operator = (const String & strSrc1) {
	setValue(strSrc1);
	return *this;
}

const String & String::operator = (const char * strSrc1) {
	setValue(strSrc1);
	return *this;
}

const String & String::operator = (const char strSrc1) {
	setValue(strSrc1);
	return *this;
}

bool String::operator == (const String & equalityTest) {
	return compareWithString(equalityTest) == 0;
}

bool String::operator == (const char * equalityTest) {
	return compareWithString(equalityTest) == 0;
}

bool String::operator != (const String & equalityTest) {
	return compareWithString(equalityTest) != 0;
}

bool String::operator != (const char * equalityTest) {
	return compareWithString(equalityTest) != 0;
}
	
String & String::operator += (const String & concatVal) {
	concatenate(concatVal);
	return *this;
}

String & String::operator += (const char * concatVal) {
	concatenate(concatVal);
	return *this;
}

String & String::operator += (const char concatVal) {
	String tmp(concatVal);
	concatenate(tmp);
	return *this;
}
	
const char String::operator [] (int Index) {
	return getCharAt(Index);
}

// Friend Overloads
	
const String operator + (const String & String1, const String & String2) {
	String tString;
	tString.concatenate(String1, String2);
	return tString;
}

////////////////////////////////////////////////////////////////////////////
// Class Body
///////////////////////////////////////

void String::init() {
	m_String = NULL;
	m_StrLen = -1;
	clearString();
}

bool String::isEmpty() {
	return (m_StrLen > 0) ? false : true;
}

bool String::clearString() {
	freeString();

	if (!reAllocToSize(1))
		return false;
	
	m_String[0] = '\0';
	m_StrLen = 0;

	return true;
}

bool String::freeString() {
	if (!m_String)
		return true;

	delete [] m_String;
	m_String = NULL;
	m_StrLen = -1;

	return true;
}

bool String::setValue(const char * data) {
	if (!data)
		return clearString();

	m_StrLen = (int) strlen(data);
	if (data == m_String)
		return true;
	if (reAllocToSize(m_StrLen + 1)) {
		strcpy(m_String, data);
		return true;
	} else
		return false;
}

bool String::setValue(const char data) {
	return setValueFromChar(data);
}

bool String::setValue(const String & Str) {
	if (!&Str)
		return clearString();

	const char * data = Str.m_String;
	m_StrLen = (int) strlen(data);
	if (reAllocToSize(m_StrLen + 1)) {
		strcpy(m_String, data);
		return true;
	} else
		return false;
}

bool String::reAllocToSize(int Size) {
	char * tString = new char [Size + 1];
	if (!tString)
		return false;
	memset(tString, 0, sizeof(char) * Size);
	if (m_String) {
		strncpy(tString, m_String, (m_StrLen < Size) ? m_StrLen : Size);
		delete [] m_String;
	}
	m_String = tString;
	
	if (m_String)
		return true;
	else
		return false;
}

int String::getLength() {
	return m_StrLen;
}

bool String::concatenate(const String & stringToAdd) {
	if (!&stringToAdd)
		return false;

	return concatenate(stringToAdd.m_String);
}

bool String::concatenate(const char * stringToAdd) {
	if (!stringToAdd)
		return true;

	int AddLen = (int) strlen(stringToAdd);
	m_StrLen += AddLen;
	if (!reAllocToSize(m_StrLen))
		return false;
	
	strcat(m_String, stringToAdd);
	return true;
}

bool String::concatenate(const String & Str1, const String & Str2) {
	setValue(Str1);
	concatenate(Str2);

	return true;
}

bool String::concatenate(const char * Str1, const String & Str2) {
	setValue(Str1);
	concatenate(Str2);

	return true;
}

void String::setToLowerCase() {
	for (int lp = 0; lp < m_StrLen; lp ++)
		m_String[lp] = tolower(m_String[lp]);
}

void String::setToUpperCase() {
	for (int lp = 0; lp < m_StrLen; lp ++)
		m_String[lp] = toupper(m_String[lp]);
}

int String::find(const char * StringToFind, bool CaseSensitive) {
	return find(String(StringToFind), CaseSensitive);
}

int String::find(const char * StringToFind, int StartPos, bool CaseSensitive) {
	return find(String(StringToFind), StartPos, CaseSensitive);
}

int String::find(const String StringToFind, bool CaseSensitive) {
	return find(StringToFind, 0, CaseSensitive);
}

int String::find(const String StringToFind, int StartPos, bool CaseSensitive) {
	if (!CaseSensitive)
		return findInsensitive(StringToFind.m_String);

	if (StartPos > m_StrLen) StartPos = m_StrLen;
	char * fString = strstr(m_String + StartPos, StringToFind.m_String);
	if (!fString)
		return - 1;
	int ret = (int) (fString - m_String);

	return (int) ret;
}

int String::findReverse(const char * StringToFind) {
	return findReverse(String(StringToFind));
}

int String::findReverse(const String StringToFind) {
	int StartPos = m_StrLen + 1;
	do {
		StartPos --;
		char * fString = strstr(m_String + StartPos, StringToFind.m_String);
		if (!fString)
			continue;

		int ret = (int) (fString - m_String);
		return ret;
	} while (StartPos > 0);

	return -1;
}

int String::findInsensitive(const char * StringToFind, int StartPos) {
	return findInsensitive(String(StringToFind), StartPos);
}

int String::findInsensitive(String StringToFind, int StartPos) {
	String tString(m_String);
	tString.setToLowerCase();

	return tString.find(StringToFind.toLower(), StartPos);
}

String String::toLower() {
	String TempString(m_String);

	TempString.setToLowerCase();

	return TempString;
}

String String::toUpper() {
	String TempString(m_String);

	TempString.setToUpperCase();

	return TempString;
}

String String::mid(int StartPos, int EndPos) {
	String Ret;

	if ( (EndPos < 0) || (EndPos >= m_StrLen) )
		EndPos = m_StrLen - 1;
		
	if (StartPos > EndPos)
		StartPos = EndPos;
	else if (StartPos < 0)
		StartPos = 0;

	int NewLen = EndPos - StartPos + 1;

	char * tString = new char[NewLen + 1];
	strncpy(tString, m_String + StartPos, NewLen);
	tString[NewLen] = '\0';

	Ret = tString;
	delete [] tString;

	return Ret;
}

bool String::rTrim() {
	int pos = m_StrLen - 1;
	while ( (pos >= 0) && (m_String[pos] == ' ') )
		pos --;
	if (pos < 0) {
		clearString();
		return true;
	}
	if (pos < m_StrLen - 1) {
		m_String[pos + 1] = '\0';
		m_StrLen = (int) strlen(m_String);
	}

	return true;
}

bool String::lTrim() {
	int count = 0;
	while ( (count < m_StrLen) && (m_String[count] == ' ') )
		count ++;
	if (count >= m_StrLen) {
		clearString();
		return true;
	}
	if (count > 0) {
		char * tString = new char[m_StrLen - count + 1];
		memset(tString, 0, sizeof(char) * m_StrLen - count + 1);
		strcpy(tString, m_String + count);
		setValue(tString);
		delete [] tString;
	}

	return true;
}

bool String::trim() {
	bool Ret = true;

	if (!lTrim())
		Ret = false;
	if (!rTrim())
		Ret = false;

	return Ret;
}

int String::stringToInt(String String) {
	return atoi(String);
}

void String::replaceSubstring(const char * StringToReplace, const String ReplaceWith, bool CaseSensitive) {
	return replaceSubstring(String(StringToReplace), ReplaceWith, CaseSensitive);
}

void String::replaceSubstring(const String StringToReplace, const char * ReplaceWith, bool CaseSensitive) {
	return replaceSubstring(StringToReplace, String(ReplaceWith), CaseSensitive);
}

void String::replaceSubstring(const char * StringToReplace, const char * ReplaceWith, bool CaseSensitive) {
	return replaceSubstring(String(StringToReplace), String(ReplaceWith), CaseSensitive);
}

void String::replaceSubstring(String StringToReplace, const String ReplaceWith, bool CaseSensitive) {
	int fPos = -1;
	do {
		if (CaseSensitive)                       	
			fPos = find(StringToReplace, fPos + 1);
		else
			fPos = findInsensitive(StringToReplace, fPos + 1);

		if (fPos == -1)
			continue;

		if (fPos == 0) {
			String tStr = mid(StringToReplace.getLength());
			setValue("");
			concatenate(ReplaceWith);
			concatenate(tStr);
		} else if (fPos == getLength() - 1) {
			String tStr = mid(0, fPos - 1);
			setValue(tStr);
		} else {
			String tStr = mid(0, fPos - 1);
			tStr += ReplaceWith;
			tStr += mid(fPos + StringToReplace.getLength());
			setValue(tStr);
		}
	} while (fPos > -1);
}

const char String::getCharAt(int Index) {
	if ( (Index < 0) || (Index >= m_StrLen) )
		return 0;
		
	return m_String[Index];
}

void String::initToCharFromInt(int IntegerValueOfChar) {
	char InitVal = char(IntegerValueOfChar);

	reAllocToSize(2);

	m_String[0] = InitVal;
	m_String[1] = '\0';
	m_StrLen = 1;
}

long String::convertToLong() {
	return atol(m_String);
}

int String::convertToInt() {
	return atoi(m_String);
}

unsigned long String::convertToULONG() {
	return strtoul(m_String, NULL, 10);
}

int String::compareWithString(const String & CompareString, bool CaseSensitive) {
	if (!CaseSensitive)
		return compareWithStringInsensitive(CompareString);
		
	return strcmp(m_String, CompareString.m_String);
}

int String::compareWithString(const char * CompareString, bool CaseSensitive) {
	if (!CaseSensitive)
		return compareWithStringInsensitive(CompareString);
		
	return strcmp(m_String, CompareString);
}

int String::compareWithStringInsensitive(const String & CompareString) {
	String c1 = m_String;
	c1.setToLowerCase();

	String c2 = CompareString;
	c2.setToLowerCase();
	return strcmp(c1.m_String, c2.m_String);
}

int String::compareWithStringInsensitive(const char * CompareString) {
	return compareWithStringInsensitive(String(CompareString));
}

bool String::setValueFromChar(char CharToSet) {
	m_StrLen = 1;
	if (reAllocToSize(m_StrLen + 1)) {
		m_String[0] = CharToSet;
		m_String[m_StrLen] = '\0';
		return true;
	} else
		return false;
}

float String::convertToFloat() {
	return (float) atof(m_String);
}

double String::convertStringToNumericalValue(bool CaseSensitive) {
	String TempString(m_String);

	if (!CaseSensitive)
		TempString.setToLowerCase();

	int Precision = m_FloatPrecision;
	if (m_FloatPrecision > TempString.getLength())
		Precision = TempString.getLength();

	double BaseVal = 1;
	double StringVal = 0;
	for (int lp = 0; lp < Precision; lp ++) {
		int CharVal = TempString[lp];
		double CharPercent = (double) CharVal / (double) 255;
		double CharsValue = CharPercent * BaseVal;

		StringVal += CharsValue;
		BaseVal *= .1;
	}

	return StringVal;
}

bool String::isNumeric() {
	for (int lp = 0; lp < getLength(); lp ++) {
		switch (m_String[lp]) {
		case '0':
		case '1':
		case '2':
		case '3':
		case '4':
		case '5':
		case '6':
		case '7':
		case '8':
		case '9':
			break;
		default:
			return false;
		}
	}

	return true;
}

bool String::beginsWith(String CompareString, bool CaseSensitive) {
	if ( (CompareString.getLength() > getLength()) || (CompareString.getLength() == 0) )
		return false;

	String tStr = mid(0, CompareString.getLength() - 1);

	if (tStr.compareWithString(CompareString, CaseSensitive) == 0)
		return true;

	return false;
}

bool String::endsWith(String CompareString, bool CaseSensitive) {
	if ( (CompareString.getLength() > getLength()) || (CompareString.getLength() == 0) )
		return false;
		
	String tStr = mid(getLength() - CompareString.getLength());

	if (tStr.compareWithString(CompareString, CaseSensitive) == 0)
		return true;

	return false;
}

void String::removeCR() {
	int pos = getLength() - 1;
	switch (m_String[pos]) {
	case 13:
	case 10:
		m_String[pos] = 0;
	}
	//printf("c = %d\n", m_String[pos]);
}

bool String::format(char * TextToOutput, ...) {
	char		text[BUFFER_SIZE];
	va_list		ap;

	va_start(ap, TextToOutput);
		vsprintf(text, TextToOutput, ap);
	va_end(ap);

	setValue(text);

	return true;
}

bool String::setCharAt(int Index, char Char) {
	if ( (Index < 0) || (Index >= getLength()) )
		return false;

	m_String[Index] = Char;
	return true;
}

