  //////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
//
// GizmoPluginAlsaMixer.cpp
//
//	GizmoPluginAlsaMixer class
//
///////////////////////////////////////
  ///////////////////////////////////////
  
/*

Copyright (c) 2005, Tim Burrell
All rights reserved.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#include "GizmoPluginAlsaMixer.hpp"
#include "libGizmo/Debug.hpp"
#include <ctype.h>

////////////////////////////////////////////////////////////////////////////
// Statics / Type Defs
///////////////////////////////////////

#define VOLUME_CALC		(int) ((m_VolumePercent / 100.0) * 255.0)
#define DEVICE_PRIORITY		128

////////////////////////////////////////////////////////////////////////////
// Overrides
///////////////////////////////////////

bool GizmoPluginAlsaMixer::onPluginOpened() {
	Debug::outDebug("GizmoPluginAlsaMixer :: onPluginOpened");
	
	return initPlugin();
}

void GizmoPluginAlsaMixer::onMixerEvent(float VolumePercent, bool Mute) {
	ScriptFuncArg * pInArg1 = m_ScriptFuncOnAlsaMixerEvent.getInArg(0);
	if (!pInArg1) {
		Debug::out("ScriptFunctionOnAlsaMixerEvent :: executeFunction -- No Parameter 1!");
		return;
	}
	
	ScriptFuncArg * pInArg2 = m_ScriptFuncOnAlsaMixerEvent.getInArg(1);
	if (!pInArg2) {
		Debug::out("ScriptFunctionOnAlsaMixerEvent :: executeFunction -- No Parameter 2!");
		return;
	}
	
	ScriptFuncArg * pInArg3 = m_ScriptFuncOnAlsaMixerEvent.getInArg(2);
	if (!pInArg3) {
		Debug::out("ScriptFunctionOnAlsaMixerEvent :: executeFunction -- No Parameter 3!");
		return;
	}
	
	ScriptFuncArg * pInArg4 = m_ScriptFuncOnAlsaMixerEvent.getInArg(3);
	if (!pInArg4) {
		Debug::out("ScriptFunctionOnAlsaMixerEvent :: executeFunction -- No Parameter 4!");
		return;
	}
	
	ScriptFuncArg * pInArg5 = m_ScriptFuncOnAlsaMixerEvent.getInArg(4);
	if (!pInArg5) {
		Debug::out("ScriptFunctionOnAlsaMixerEvent :: executeFunction -- No Parameter 5!");
		return;
	}
	
	pInArg1->ArgNumber = m_DeviceID;
	pInArg2->ArgNumber = getVolumePercent();
	pInArg3->ArgBoolean = getSwitch();
	pInArg4->ArgString = m_DeviceCard;
	pInArg5->ArgString = m_DeviceMixer;
	
	if (m_pScriptEngine)
		m_pScriptEngine->callScriptFunction(&m_ScriptFuncOnAlsaMixerEvent);
}

////////////////////////////////////////////////////////////////////////////
// Callbacks
///////////////////////////////////////

bool GizmoPluginAlsaMixer::checkEventsWaiting() {
	snd_mixer_handle_events(m_Handle);
	return true;
}

int GizmoPluginAlsaMixer::mixerCallbackRefresh(snd_mixer_elem_t * elem, unsigned int mask) {
	LinkList<AlsaMixerInterface> * pAlsaMixerList = AlsaMixerInterface::getPluginList();
	if (pAlsaMixerList->getSize() == 0)
		return 0;

	pAlsaMixerList->moveToHead();
	do {
		AlsaMixerInterface * pMixer = pAlsaMixerList->getData();
		if (pMixer->getMixerElem() == elem) {
			pMixer->getVolumeFromCard();
			pMixer->onMixerEvent(pMixer->getVolumePercent(), !pMixer->getSwitch());
			break;
		}
	} while (pAlsaMixerList->moveNext());
		
	return 0;
}

////////////////////////////////////////////////////////////////////////////
// Construction
///////////////////////////////////////

GizmoPluginAlsaMixer::GizmoPluginAlsaMixer(ScriptEngineInterface * pScriptEngine, const char * DevicePath) : 
	m_ScriptFuncAlsaMixerGetVolume(this),
	m_ScriptFuncAlsaMixerSetVolume(this),
	m_ScriptFuncAlsaMixerGetMute(this),
	m_ScriptFuncAlsaMixerSetMute(this),
	m_ScriptFuncAlsaMixerGetMixerName(this)
{
	m_pScriptEngine = pScriptEngine;
	
	String strDPath = DevicePath;
	int sPos = strDPath.find(";");
	if (sPos < 0) {
		Debug::out("GizmoPluginAlsaMixer :: Invalid Initializer [%s] -- Need SemiColon Separated List!", DevicePath);
		return;
	}
	m_DeviceCard = strDPath.mid(0, sPos - 1);
	m_DeviceMixer = strDPath.mid(sPos + 1);
	m_WatchingDevice = false;
	
	m_Handle = NULL;
	m_SID = NULL;
	m_Element = NULL;
	m_HWInfo = NULL;
	m_CTLHandle = NULL;
	m_ElemCount = -1;
	
	Debug::outDebug("GizmoPluginAlsaMixer :: AlsaMixer Input Plugin Loading [%s @ %s]...", m_DeviceCard.getString(), m_DeviceMixer.getString());
}

GizmoPluginAlsaMixer::~GizmoPluginAlsaMixer() {
	closePlugin();
	Debug::outDebug("GizmoPluginAlsaMixer :: AlsaMixer Input Plugin Unloaded.");
}

////////////////////////////////////////////////////////////////////////////
// Class Body
///////////////////////////////////////

void GizmoPluginAlsaMixer::closePlugin() {
	Debug::outDebug("GizmoPluginAlsaMixer :: AlsaMixer Input Plugin Closing...");
	
	m_WatchingDevice = false;
	closeAlsa();
	closeMixer();

	Debug::outDebug("GizmoPluginAlsaMixer :: AlsaMixer Input Plugin Closed");
}

bool GizmoPluginAlsaMixer::initPlugin() {
	Debug::outDebug("    AlsaMixer Input Plugin v%s [%s @ %s]", PACKAGE_VERSION, m_DeviceCard.getString(), m_DeviceMixer.getString());

	if (!m_pScriptEngine) {
		Debug::outError("GizmoPluginAlsaMixer :: Invalid Script Engine!");
		return false;
	}

	// register functions
	m_pScriptEngine->registerScriptFunction(&m_ScriptFuncAlsaMixerGetVolume);
	m_pScriptEngine->registerScriptFunction(&m_ScriptFuncAlsaMixerSetVolume);
	m_pScriptEngine->registerScriptFunction(&m_ScriptFuncAlsaMixerGetMute);
	m_pScriptEngine->registerScriptFunction(&m_ScriptFuncAlsaMixerSetMute);
	m_pScriptEngine->registerScriptFunction(&m_ScriptFuncAlsaMixerGetMixerName);
	
	// initialize
	bool ret = openMixer(m_DeviceCard, m_DeviceMixer);
	
	Debug::out("\t- Found [%s] at [%s]", m_DeviceCard.getString(), m_DeviceMixer.getString());
	Debug::outDebug("GizmoPluginAlsaMixer :: AlsaMixer Input Plugin Initialized.");
	
	return ret;
}

void GizmoPluginAlsaMixer::closeAlsa() {
	m_WatchingDevice = false;
}

bool GizmoPluginAlsaMixer::openMixer(const char * Card, const char * Mixer) {
	closeMixer();
	
	m_Card = Card;
	m_Mixer = Mixer;

	//////////////////////

	snd_ctl_card_info_alloca(&m_HWInfo);
	
	int err;
	if ((err = snd_ctl_open(&m_CTLHandle, m_Card, 0)) < 0) {
		Debug::outError("GizmoPluginAlsaMixer :: Mixer %s CTL open error: %s", m_Card.getString(), snd_strerror(err));
		return false;
	}

	if ((err = snd_ctl_card_info(m_CTLHandle, m_HWInfo)) < 0) {
		Debug::outError("GizmoPluginAlsaMixer :: Mixer %s CTL HW Info error: %s", m_Card.getString(), snd_strerror(err));
		return false;
	}

	snd_ctl_close(m_CTLHandle);
	m_CTLHandle = NULL;

	////////////////////

	// parse the SID
	snd_mixer_selem_id_alloca(&m_SID);
	if (parseSimpleID()) {
		Debug::outError("GizmoPluginAlsaMixer :: Wrong %s identifier: %s", m_Card.getString(), m_Mixer.getString());
		return false;
	}
	
	if ((err = snd_mixer_open(&m_Handle, 0)) < 0) {
		Debug::outError("GizmoPluginAlsaMixer :: Mixer %s open error: %s", m_Card.getString(), snd_strerror(err));
		return false;
	}
	
	if ((err = snd_mixer_attach(m_Handle, m_Card.getString())) < 0) {
		Debug::outError("GizmoPluginAlsaMixer :: Mixer attach %s error: %s", m_Card.getString(), snd_strerror(err));
		closeMixer();
		return false;
	}

	if ((err = snd_mixer_selem_register(m_Handle, NULL, NULL)) < 0) {
		Debug::outError("GizmoPluginAlsaMixer :: Mixer register %s error: %s", m_Card.getString(), snd_strerror(err));
		closeMixer();
		return false;
	}
	
	if ((err = snd_mixer_load(m_Handle)) < 0) {
		Debug::outError("GizmoPluginAlsaMixer :: Mixer load %s error: %s", m_Card.getString(), snd_strerror(err));
		closeMixer();
		return false;
	}	

	if ((m_Element = snd_mixer_find_selem(m_Handle, m_SID)) == NULL) {
		Debug::outError("GizmoPluginAlsaMixer :: Unable to find simple control for %s", m_Mixer.getString());
		closeMixer();
		return false;
	}

	// register the callback
	snd_mixer_elem_set_callback(m_Element, mixerCallbackRefresh);

	// get volume
	getVolumeFromCard();
		
	Debug::out("\t- [%s] on [%s]\n\t- Watching Mixer Element [%s]\n\t- (Volume Currently %.1f%%) [mute: %s]", snd_ctl_card_info_get_name(m_HWInfo), snd_ctl_card_info_get_mixername(m_HWInfo), m_Mixer.getString(), m_VolumePercent, m_Switch ? "off" : "on");
	
	return true;
}

void GizmoPluginAlsaMixer::getVolumeFromCard() {
	snd_mixer_selem_get_playback_volume_range(m_Element, &m_VolumeMin, &m_VolumeMax);
	snd_mixer_selem_get_playback_volume(m_Element, SND_MIXER_SCHN_FRONT_LEFT, &m_VolumeCur);

	if ( (m_HasSwitch = snd_mixer_selem_has_playback_switch(m_Element)) )
		snd_mixer_selem_get_playback_switch(m_Element, SND_MIXER_SCHN_FRONT_LEFT, &m_Switch);

	m_VolumePercent = (float)m_VolumeCur / (float)m_VolumeMax * 100.0f;
}

bool GizmoPluginAlsaMixer::setVolume(long Value) {
	if (!m_Element)
		return false;

	if (Value > m_VolumeMax)
		Value = m_VolumeMax;
	else if (Value < 0)
		Value = 0;

	int MessageData = 0;
	if ( ((m_VolumeCur == m_VolumeMax) && (Value == m_VolumeCur)) ||
	     ((m_VolumeCur == 0) && (Value == 0)) ) {
		MessageData = -1;
	}
	m_VolumeCur = Value;
	m_VolumePercent = (float)m_VolumeCur / (float)m_VolumeMax * 100.0f;

	snd_mixer_selem_set_playback_volume_all(m_Element, m_VolumeCur);

	Debug::outDebug("GizmoPluginAlsaMixer :: Volume now at [%.1f%%] (%d)", m_VolumePercent, m_VolumeCur);

	return true;
}

bool GizmoPluginAlsaMixer::setVolumePercent(float Value) {
	if (!m_Element)
		return false;
		
	if (Value > 100.0f)
		Value = 100.0f;
	else if (Value < 0.0f)
		Value = 0.0f;
		
	m_VolumePercent = Value;
	m_VolumeCur = (long) (float(m_VolumeMax - m_VolumeMin) * (Value / 100)) + m_VolumeMin;

	snd_mixer_selem_set_playback_volume_all(m_Element, m_VolumeCur);
	
	Debug::outDebug("GizmoPluginAlsaMixer :: Volume now at [%.1f%%] (%d)", m_VolumePercent, m_VolumeCur);

	return true;
}

float GizmoPluginAlsaMixer::getVolumePercent() {
	return m_VolumePercent;
}
	
long GizmoPluginAlsaMixer::getVolume() {
	return m_VolumeCur;
}

bool GizmoPluginAlsaMixer::incrementVolumeReal(long Value) {
	return setVolume(m_VolumeCur + Value);
}

bool GizmoPluginAlsaMixer::decrementVolumeReal(long Value) {
	return setVolume(m_VolumeCur - Value);
}

bool GizmoPluginAlsaMixer::incrementVolumePercent(float Value) {
	return setVolumePercent(m_VolumePercent + Value);
}

bool GizmoPluginAlsaMixer::decrementVolumePercent(float Value) {
	return setVolumePercent(m_VolumePercent - Value);
}

bool GizmoPluginAlsaMixer::setSwitch(int Value) {
	if ( (Value < 0) || (Value > 1) )
		return false;

	m_Switch = Value ? 0 : 1;
	snd_mixer_selem_set_playback_switch_all(m_Element, m_Switch);

	return true;
}

int GizmoPluginAlsaMixer::getSwitch() {
	return ! m_Switch;
}

bool GizmoPluginAlsaMixer::toggleSwitch() {
	return setSwitch(m_Switch ? 0 : 1);
}

void GizmoPluginAlsaMixer::closeMixer() {
	m_VolumeMin = -1;
	m_VolumeMax = -1;
	m_VolumeCur = -1;
	m_Switch = -1;
	m_ElemCount = -1;

	m_SID = NULL;
	m_Element = NULL;
	m_HWInfo = NULL;
	m_CTLHandle = NULL;

	if (m_Handle) {
		snd_mixer_close(m_Handle);
		m_Handle = NULL;
	}
}

/*
 * Taken from amixer.c (from alsa-utils-1.0.6)
 *
 *  Thanks alsa people.
 */
int GizmoPluginAlsaMixer::parseSimpleID() {
	int c, size;
	char buf[128];
	char *ptr = buf;
	const char *str = m_Mixer.getString();

	while (*str == ' ' || *str == '\t')
		str++;
	if (!(*str))
		return -EINVAL;
	size = 1;	/* for '\0' */
	if (*str != '"' && *str != '\'') {
		while (*str && *str != ',') {
			if (size < (int)sizeof(buf)) {
				*ptr++ = *str;
				size++;
			}
			str++;
		}
	} else {
		c = *str++;
		while (*str && *str != c) {
			if (size < (int)sizeof(buf)) {
				*ptr++ = *str;
				size++;
			}
			str++;
		}
		if (*str == c)
			str++;
	}
	if (*str == '\0') {
		snd_mixer_selem_id_set_index(m_SID, 0);
		*ptr = 0;
		goto _set;
	}
	if (*str != ',')
		return -EINVAL;
	*ptr = 0;	/* terminate the string */
	str++;
	if (!isdigit(*str))
		return -EINVAL;
	snd_mixer_selem_id_set_index(m_SID, atoi(str));
       _set:
	snd_mixer_selem_id_set_name(m_SID, buf);
	return 0;
}

String GizmoPluginAlsaMixer::getDeviceName() {
	String retStr;
	retStr.format("AlsaMixer (%s)", m_Card.getString());
	return retStr;
}

String GizmoPluginAlsaMixer::getDeviceNameAlt() {
	return m_Mixer;
}

snd_mixer_elem_t * GizmoPluginAlsaMixer::getMixerElem() {
	return m_Element;
}
