  //////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
//
// ScriptFunctionAlsaMixerSetMute.cpp
//
//	ScriptFunctionAlsaMixerSetMute class
//
///////////////////////////////////////
  ///////////////////////////////////////
  
/*

Copyright (c) 2005, Tim Burrell
All rights reserved.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#include "ScriptFunctionAlsaMixerSetMute.hpp"
#include "libGizmo/Debug.hpp"

////////////////////////////////////////////////////////////////////////////
// Statics / Type Defs
///////////////////////////////////////

////////////////////////////////////////////////////////////////////////////
// Overrides
///////////////////////////////////////

bool ScriptFunctionAlsaMixerSetMute::executeFunction() {
	ScriptFuncArg * pInArg1 = getInArg(0);
	if (!pInArg1) {
		Debug::out("ScriptFunctionAlsaMixerSetMute :: executeFunction -- No In Parameter!");
		return false;
	}
	
	ScriptFuncArg * pInArg2 = getInArg(1);
	if (!pInArg2) {
		Debug::out("ScriptFunctionAlsaMixerSetMute :: executeFunction -- No In Parameter 2!");
		return false;
	}
	
	ScriptFuncArg * pOutArg1 = getOutArg(0);
	if (!pInArg1) {
		Debug::out("ScriptFunctionAlsaMixerSetMute :: executeFunction -- No Out Parameter!");
		return false;
	}
	
	// find device
	LinkList<AlsaMixerInterface> * pPluginList = AlsaMixerInterface::getPluginList();
	if (pPluginList->getSize() == 0) {
		Debug::out("ScriptFunctionAlsaMixerSetMute :: PluginList Badness!");
		return false;
	}
	pPluginList->moveToHead();
	do {
		AlsaMixerInterface * pInt = pPluginList->getData();
		if (pInt->getDeviceID() == pInArg1->ArgNumber) {
			pOutArg1->ArgBoolean = pInt->setSwitch((int)pInArg2->ArgBoolean);
			break;
		}
	} while (pPluginList->moveNext());
	
	Debug::outDebug("ScriptFunctionAlsaMixerSetMute :: Mute Set to [%s]", pInArg2->ArgBoolean ? "true" : "false");
		
	return true;
}

////////////////////////////////////////////////////////////////////////////
// Global Funcs
///////////////////////////////////////

////////////////////////////////////////////////////////////////////////////
// Construction
///////////////////////////////////////

ScriptFunctionAlsaMixerSetMute::ScriptFunctionAlsaMixerSetMute(AlsaMixerInterface * pPlugin) {
	m_pPlugin = pPlugin;
	FunctionName.format("alsaMixerSetMute");
	FunctionDescription = "Sets the Mute Status of the Alsa Mixer";
	
	addInArg(SCRIPT_ARG_NUMBER);		// IN  DeviceID
	addInArg(SCRIPT_ARG_BOOLEAN);		// IN  Mute
	addOutArg(SCRIPT_ARG_BOOLEAN);		// OUT did it work?
}

ScriptFunctionAlsaMixerSetMute::~ScriptFunctionAlsaMixerSetMute() {
}

////////////////////////////////////////////////////////////////////////////
// Class Body
///////////////////////////////////////
