  //////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
//
// GizmoPluginBMPControl.cpp
//
//	GizmoPluginBMPControl class
//
///////////////////////////////////////
  ///////////////////////////////////////
  
/*

Copyright (c) 2005, Tim Burrell
All rights reserved.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#include "GizmoPluginBMPControl.hpp"
#include "libGizmo/Debug.hpp"

////////////////////////////////////////////////////////////////////////////
// Statics / Type Defs
///////////////////////////////////////

////////////////////////////////////////////////////////////////////////////
// Statics / Globals
///////////////////////////////////////

////////////////////////////////////////////////////////////////////////////
// Overrides
///////////////////////////////////////

bool GizmoPluginBMPControl::onPluginOpened() {
	Debug::outDebug("GizmoPluginBMPControl :: onPluginOpened");
	
	return initPlugin();
}

////////////////////////////////////////////////////////////////////////////
// Construction
///////////////////////////////////////

GizmoPluginBMPControl::GizmoPluginBMPControl(ScriptEngineInterface * pScriptEngine, const char * DevicePath) :
	m_ScriptFuncBMPGetOutputTime(this),
	m_ScriptFuncBMPGetPlaylistPosition(this),
	m_ScriptFuncBMPIsPaused(this),
	m_ScriptFuncBMPIsPlaying(this),
	m_ScriptFuncBMPNextTrack(this),
	m_ScriptFuncBMPPause(this),
	m_ScriptFuncBMPPlay(this),
	m_ScriptFuncBMPPreviousTrack(this),
	m_ScriptFuncBMPSeek(this),
	m_ScriptFuncBMPSetTrack(this),
	m_ScriptFuncBMPStop(this)
{
	Debug::outDebug("GizmoPluginBMPControl :: BMPControl Output Plugin Loading...");

	m_pScriptEngine = pScriptEngine;
	m_DeviceName = "OutputBMPControl";
}

GizmoPluginBMPControl::~GizmoPluginBMPControl() {
	closePlugin();
	Debug::outDebug("GizmoPluginBMPControl :: BMPControl Output Plugin Unloaded.");
}

////////////////////////////////////////////////////////////////////////////
// Class Body
///////////////////////////////////////

void GizmoPluginBMPControl::closePlugin() {
	Debug::outDebug("GizmoPluginBMPControl :: BMPControl Output Plugin Closing...");
	closeBMPControl();
}

bool GizmoPluginBMPControl::initPlugin() {
	Debug::out("        - BMPControl Output Plugin v%s", PACKAGE_VERSION);

	if (!m_pScriptEngine) {
		Debug::outError("GizmoPluginBMPControl :: Invalid Script Engine!");
		return false;
	}
	
	// register functions
	Debug::outDebug("GizmoPluginBMPControl :: Registering Functions with Script Engine...");
	m_pScriptEngine->registerScriptFunction(&m_ScriptFuncBMPGetOutputTime);
	m_pScriptEngine->registerScriptFunction(&m_ScriptFuncBMPGetPlaylistPosition);
	m_pScriptEngine->registerScriptFunction(&m_ScriptFuncBMPIsPaused);
	m_pScriptEngine->registerScriptFunction(&m_ScriptFuncBMPIsPlaying);
	m_pScriptEngine->registerScriptFunction(&m_ScriptFuncBMPNextTrack);
	m_pScriptEngine->registerScriptFunction(&m_ScriptFuncBMPPause);
	m_pScriptEngine->registerScriptFunction(&m_ScriptFuncBMPPlay);
	m_pScriptEngine->registerScriptFunction(&m_ScriptFuncBMPPreviousTrack);
	m_pScriptEngine->registerScriptFunction(&m_ScriptFuncBMPSeek);
	m_pScriptEngine->registerScriptFunction(&m_ScriptFuncBMPSetTrack);
	m_pScriptEngine->registerScriptFunction(&m_ScriptFuncBMPStop);
	
	Debug::outDebug("GizmoPluginBMPControl :: BMPControl Output Plugin Initialized.");
	return true;
}

String GizmoPluginBMPControl::getDeviceName() {
	return "GizmoPluginBMPControl";
}

String GizmoPluginBMPControl::getDeviceNameAlt() {
	return "BMPControl";
}

int GizmoPluginBMPControl::bmpGetPlaylistPosition() {
	return BMPGetPlaylistPosition();
}

int GizmoPluginBMPControl::bmpGetOutputTime() {
	return BMPGetOutputTime();
}

bool GizmoPluginBMPControl::bmpIsPaused() {
	return BMPIsPaused();
}

bool GizmoPluginBMPControl::bmpIsPlaying() {
	return BMPIsPlaying();
}

bool GizmoPluginBMPControl::bmpNextTrack() {
	return BMPNextTrack();
}

bool GizmoPluginBMPControl::bmpPause() {
	return BMPPause();
}

bool GizmoPluginBMPControl::bmpPlay() {
	return BMPPlay();
}

bool GizmoPluginBMPControl::bmpPreviousTrack() {
	return BMPPreviousTrack();
}

bool GizmoPluginBMPControl::bmpSeek(int Offset) {
	return BMPSeek(Offset);
}

bool GizmoPluginBMPControl::bmpSetTrack(int Track) {
	return BMPSetTrack(Track);
}

bool GizmoPluginBMPControl::bmpStop() {
	return BMPStop();
}
