  ///////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////
//
// GizmoPluginBMPControl.hpp
//
//	GizmoPluginBMPControl header file
//
///////////////////////////////////////
  ///////////////////////////////////////
  
/*

Copyright (c) 2005, Tim Burrell
All rights reserved.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#ifndef __GizmoPluginBMPControl_h
#define __GizmoPluginBMPControl_h

#if HAVE_CONFIG_H
#include "config.h"
#endif

#include "libGizmo/ScriptEngineInterface.hpp"
#include "libGizmo/String.hpp"
#include "libGizmo/Debug.hpp"
#include "BMPControlInterface.hpp"
#include "BMPControl.hpp"
#include "ScriptFunctionBMPPlay.hpp"
#include "ScriptFunctionBMPGetOutputTime.hpp"
#include "ScriptFunctionBMPGetPlaylistPosition.hpp"
#include "ScriptFunctionBMPIsPaused.hpp"
#include "ScriptFunctionBMPIsPlaying.hpp"
#include "ScriptFunctionBMPNextTrack.hpp"
#include "ScriptFunctionBMPPause.hpp"
#include "ScriptFunctionBMPPlay.hpp"
#include "ScriptFunctionBMPPreviousTrack.hpp"
#include "ScriptFunctionBMPSeek.hpp"
#include "ScriptFunctionBMPSetTrack.hpp"
#include "ScriptFunctionBMPStop.hpp"

//////////////////////////////////////////////////////////////////////////////
// Typedefs
///////////////////////////////////////
	
//////////////////////////////////////////////////////////////////////////////
// Class Definition
///////////////////////////////////////

class GizmoPluginBMPControl : public BMPControlInterface, public BMPControl {
public:
	void 		closePlugin();
	String		getDeviceName();
	String		getDeviceNameAlt();
	bool		initPlugin();
	virtual bool	onPluginOpened();
	virtual int 	bmpGetPlaylistPosition();
	virtual int	bmpGetOutputTime();
	virtual bool	bmpIsPaused();
	virtual bool	bmpIsPlaying();
	virtual bool	bmpNextTrack();
	virtual bool	bmpPause();
	virtual bool	bmpPlay();
	virtual bool	bmpPreviousTrack();
	virtual bool	bmpSeek(int Offset);
	virtual bool	bmpSetTrack(int Track);
	virtual bool	bmpStop();

	GizmoPluginBMPControl(ScriptEngineInterface * pScriptEngine, const char * DevicePath);
	virtual ~GizmoPluginBMPControl();
		
private:
	ScriptEngineInterface * m_pScriptEngine;

	ScriptFunctionBMPPlay		m_ScriptFuncBMPPlay;
	ScriptFunctionBMPGetOutputTime	m_ScriptFuncBMPGetOutputTime;
	ScriptFunctionBMPGetPlaylistPosition m_ScriptFuncBMPGetPlaylistPosition;
	ScriptFunctionBMPIsPaused	m_ScriptFuncBMPIsPaused;
	ScriptFunctionBMPIsPlaying	m_ScriptFuncBMPIsPlaying;
	ScriptFunctionBMPNextTrack	m_ScriptFuncBMPNextTrack;
	ScriptFunctionBMPPause		m_ScriptFuncBMPPause;
	ScriptFunctionBMPPreviousTrack	m_ScriptFuncBMPPreviousTrack;
	ScriptFunctionBMPSeek		m_ScriptFuncBMPSeek;
	ScriptFunctionBMPSetTrack	m_ScriptFuncBMPSetTrack;
	ScriptFunctionBMPStop 		m_ScriptFuncBMPStop;

};

//////////////////////////////////////////////////////////////////////////////
// Shared Object Attach Function
///////////////////////////////////////

extern "C" {
	GizmoPlugin * getPlugin(LinkList<void> * pArgList) {
		ScriptEngineInterface * pScriptEngine;
		if ((pScriptEngine = (ScriptEngineInterface *) pArgList->getDataFromIndex(0)) == NULL) {
			Debug::outError("GizmoPluginCPUUsage :: Invalid Call to getPlugin -- No ScriptEngine!");
			return NULL;
		}
		
		String * pStrDevicePath;
		if ((pStrDevicePath = (String *) pArgList->getDataFromIndex(1)) == NULL) {
			Debug::outError("GizmoPluginCPUUsage :: Invalid Call to getPlugin -- No DevicePath!");
			return NULL;
		}
		
		return new GizmoPluginBMPControl(pScriptEngine, *pStrDevicePath);
	}
}

#endif // __GizmoPluginBMPControl_h

