  //////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
//
// CPUUsage.cpp
//
//	CPUUsage class
//
///////////////////////////////////////
  ///////////////////////////////////////
  
/*

Copyright (c) 2005, Tim Burrell
All rights reserved.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#include "CPUUsage.hpp"
#include "libGizmo/Debug.hpp"
#include "libGizmo/UtilTime.hpp"

#include <unistd.h>
#include <stdio.h>
#include <errno.h>
#include <string.h>

////////////////////////////////////////////////////////////////////////////
// Globals, Defines, Type Defs
///////////////////////////////////////

#define STAT_FILE	"/proc/stat"

////////////////////////////////////////////////////////////////////////////
// Overloads
///////////////////////////////////////

void CPUUsage::onCPUUsage(unsigned char Usage, float UsagePercent) {
	Debug::outDebug("CPUUsage :: Usage == %.2lf [%d]", UsagePercent, Usage);
}

////////////////////////////////////////////////////////////////////////////
// Construction
///////////////////////////////////////

CPUUsage::CPUUsage() : m_Average(10) {
	m_File = NULL;
	m_Delay = 50000;
	m_UsagePercent = 0.0f;
	m_UsageReal = 0;
	m_Twitchy = false;
	m_LastTime = UtilTime::getTicks();
}

CPUUsage::~CPUUsage() {
	if (m_File) {
		fclose(m_File);
		m_File = NULL;
	}
}

////////////////////////////////////////////////////////////////////////////
// Class Body
///////////////////////////////////////

void CPUUsage::startMonitoringCPU() {
}

/*
 * Thanks goes to author of asmon applet for help with this
 */
bool CPUUsage::checkEventsWaiting() {
	if (UtilTime::getTicks() - m_LastTime < (unsigned long) m_Delay)	
		return false;

	static double cpuInfo[7] = {0}, cpuStat[7] = {0}, cpuFields[7] = {0};
	static double cpuTotal = 0;

	// open the file
	if ((m_File = fopen(STAT_FILE, "r")) == NULL) {
		Debug::outError("CPUUsage :: Failed to open Stat File [%s] : %s", STAT_FILE, strerror(errno));
		return false;
	}

	// read and close
	fscanf(m_File, "cpu %lf %lf %lf %lf %lf %lf %lf", cpuInfo, cpuInfo+1, cpuInfo+2, cpuInfo+3, cpuInfo+4, cpuInfo+5, cpuInfo+6);
	fclose(m_File);
	m_File = NULL;

	// Calculate CPU stuff
	cpuTotal = 0.0;
	for (int lp = 0; lp < 7; lp ++) {
		cpuFields[lp] = cpuInfo[lp] - cpuStat[lp];
		cpuTotal += cpuFields[lp];
		cpuStat[lp] = cpuInfo[lp];
	}

	// fields 3 and 4 are idle time, and iowait time
	// kernel is pre-emptable during iowait, so it's okay to subtract
	if (m_Twitchy)
		m_UsagePercent = (cpuTotal - cpuFields[3] - cpuFields[4]) / cpuTotal;
	else {
		m_Average.push( (cpuTotal - cpuFields[3] - cpuFields[4]) / cpuTotal );
		m_UsagePercent = m_Average.average();
	}
	
	m_UsageReal = (unsigned char) (m_UsagePercent * 255.0);
	m_UsagePercent *= 100.0f;

	//Debug::outDebug("CPUUsage :: Usage == %.2lf [%d]", m_UsagePercent, m_UsageReal);
	onCPUUsage(m_UsageReal, m_UsagePercent);
	m_LastTime = UtilTime::getTicks();

	return true;
}

void CPUUsage::setCPUPollDelay(int Delay) {
	m_Delay = Delay;
	Debug::outDebug("CPUUsage :: Set CPU Poll Delay [%d]", Delay);
}

int CPUUsage::getCPUPollDelay() {
	return m_Delay;
}

unsigned char CPUUsage::getCPUUsage() {
	return m_UsageReal;
}

float CPUUsage::getCPUUsagePercent() {
	return m_UsagePercent;
}

void CPUUsage::setCPUTwitchy(bool Twitch) {
	m_Twitchy = Twitch;
}

