  //////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
//
// GizmoPluginCPUUsage.cpp
//
//	GizmoPluginCPUUsage class
//
///////////////////////////////////////
  ///////////////////////////////////////
  
/*

Copyright (c) 2005, Tim Burrell
All rights reserved.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#include "GizmoPluginCPUUsage.hpp"
#include "libGizmo/Debug.hpp"
#include "libGizmo/UtilTime.hpp"

////////////////////////////////////////////////////////////////////////////
// Statics / Type Defs
///////////////////////////////////////

////////////////////////////////////////////////////////////////////////////
// Overrides
///////////////////////////////////////

void GizmoPluginCPUUsage::onCPUUsage(unsigned char Usage, float UsagePercent) {
	const int IN_ARGS = 2;
	
	if (m_pScriptEngine == NULL)
		return;

	ScriptFuncArg * pInArg[IN_ARGS];
	int lp;
	
	for (lp = 0; lp < IN_ARGS; lp ++) {
		pInArg[lp] = m_ScriptFuncOnCPUUsage.getInArg(lp);
		if (!pInArg[lp]) {
			Debug::outDebug("GizmoPluginGeneric :: Failed to Call OnGenericRelease -- Arg %d", lp);
			return;
		}
	}
		
	pInArg[0]->ArgNumber = Usage;
	pInArg[1]->ArgNumber = UsagePercent;
		
	m_pScriptEngine->callScriptFunction(&m_ScriptFuncOnCPUUsage);		
}

bool GizmoPluginCPUUsage::onPluginOpened() {
	Debug::outDebug("GizmoPluginCPUUsage :: onPluginOpened");
	
	return initPlugin();
}

////////////////////////////////////////////////////////////////////////////
// Construction
///////////////////////////////////////

GizmoPluginCPUUsage::GizmoPluginCPUUsage(ScriptEngineInterface * pScriptEngine, const char * DevicePath) {
	Debug::outDebug("GizmoPluginCPUUsage :: CPU Montor Plugin Loading...");
	
	m_pScriptEngine = pScriptEngine;
	m_DevicePath = DevicePath;
	m_DeviceName = "GizmoPluginCPUUsage";
}

GizmoPluginCPUUsage::~GizmoPluginCPUUsage() {
	closePlugin();
	Debug::outDebug("GizmoPluginCPUUsage :: CPU Monitor Plugin Unloaded.");
}

////////////////////////////////////////////////////////////////////////////
// Class Body
///////////////////////////////////////

void GizmoPluginCPUUsage::closePlugin() {
	Debug::outDebug("GizmoPluginCPUUsage :: CPU Monitor Plugin Closing...");
}

bool GizmoPluginCPUUsage::initPlugin() {
	Debug::out("        - CPU Monitor Plugin v%s [%s]", PACKAGE_VERSION, m_DevicePath.getString());

	if (!m_pScriptEngine) {
		Debug::outError("GizmoPluginCPUUsage :: Invalid Script Engine!");
		return false;
	}
	
	// register functions

	startMonitoringCPU();
	Debug::outDebug("GizmoPluginCPUUsage :: CPU Monitor Initialized.");
	return true;
}

String GizmoPluginCPUUsage::getDeviceName() {
	return m_DeviceName;
}

String GizmoPluginCPUUsage::getDeviceNameAlt() {
	return m_DevicePath;
}

void GizmoPluginCPUUsage::scriptFunction_SetCPUDelay(ScriptFunction * pFunc) {
	if (!ScriptFunction::checkScriptFunc(pFunc))
		return;

	GizmoPluginCPUUsage * pThis = (GizmoPluginCPUUsage *) pFunc->Arguments;

	// get argument
	ScriptFuncArg * pArg = NULL;
	if (pFunc->InArgList.getSize() > 0) {
		pArg = pFunc->InArgList.getDataFromIndex(0);
		if ( (!pArg) || (pArg->Type != SCRIPT_ARG_NUMBER) )
			return;
	}

	// set sensitivity
	pThis->m_Delay = (int) pArg->ArgNumber;
	if (pThis->m_Delay < 10000)
	    pThis->m_Delay = 10000;

	Debug::outDebug("GizmoPluginCPUUsage :: CPU Delay Set to [%d]", pThis->m_Delay);
	    
	// return true
	pArg = new ScriptFuncArg();
	pArg->Type = SCRIPT_ARG_BOOLEAN;
	pArg->ArgBoolean = true;
	pFunc->OutArgList.addTail(pArg);
}

void GizmoPluginCPUUsage::scriptFunction_SetCPUTwitchy(ScriptFunction * pFunc) {
	if (!ScriptFunction::checkScriptFunc(pFunc))
		return;

	GizmoPluginCPUUsage * pThis = (GizmoPluginCPUUsage *) pFunc->Arguments;

	// get argument
	ScriptFuncArg * pArg = NULL;
	if (pFunc->InArgList.getSize() > 0) {
		pArg = pFunc->InArgList.getDataFromIndex(0);
		if ( (!pArg) || (pArg->Type != SCRIPT_ARG_BOOLEAN) )
			return;
	}

	// set sensitivity
	pThis->m_Twitchy = pArg->ArgBoolean;

	Debug::outDebug("GizmoPluginCPUUsage :: CPU Twitchy Set to [%s]", pThis->m_Twitchy ? "true" : "false");
	    
	// return true
	pArg = new ScriptFuncArg();
	pArg->Type = SCRIPT_ARG_BOOLEAN;
	pArg->ArgBoolean = true;
	pFunc->OutArgList.addTail(pArg);
}

bool GizmoPluginCPUUsage::checkEventsWaiting() {
	return CPUUsage::checkEventsWaiting();
}
