  ///////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////
//
// GizmoPluginCPUUsage.hpp
//
//	GizmoPluginCPUUsage header file
//
///////////////////////////////////////
  ///////////////////////////////////////
  
/*

Copyright (c) 2005, Tim Burrell
All rights reserved.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#ifndef __GizmoPluginCPUUsage_h
#define __GizmoPluginCPUUsage_h

#if HAVE_CONFIG_H
#include "config.h"
#endif

#include "CPUUsage.hpp"
#include "libGizmo/Debug.hpp"
#include "libGizmo/String.hpp"
#include "libGizmo/ScriptEngineInterface.hpp"
#include "CPUUsageInterface.hpp"
#include "ScriptFunctionOnCPUUsage.hpp"

//////////////////////////////////////////////////////////////////////////////
// Typedefs
///////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
// Class Definition
///////////////////////////////////////

class GizmoPluginCPUUsage : public CPUUsageInterface, public CPUUsage {
public:
	bool		checkEventsWaiting();
	void 		closePlugin();
	String		getDeviceName();
	String		getDeviceNameAlt();
	bool		initPlugin();
	void 		onCPUUsage(unsigned char Usage, float UsagePercent);
	virtual bool	onPluginOpened();

	GizmoPluginCPUUsage(ScriptEngineInterface * pScriptEngine, const char * DevicePath);
	virtual ~GizmoPluginCPUUsage();
	
	static void 	scriptFunction_SetCPUDelay(ScriptFunction * pFunc);
	static void 	scriptFunction_SetCPUTwitchy(ScriptFunction * pFunc);
	
private:
	ScriptEngineInterface * m_pScriptEngine;
	
	ScriptFunctionOnCPUUsage	m_ScriptFuncOnCPUUsage;
};

//////////////////////////////////////////////////////////////////////////////
// Shared Object Attach Function
///////////////////////////////////////

extern "C" {
	GizmoPlugin * getPlugin(LinkList<void> * pArgList) {
		ScriptEngineInterface * pScriptEngine;
		if ((pScriptEngine = (ScriptEngineInterface *) pArgList->getDataFromIndex(0)) == NULL) {
			Debug::outError("GizmoPluginCPUUsage :: Invalid Call to getPlugin -- No ScriptEngine!");
			return NULL;
		}
		
		String * pStrDevicePath;
		if ((pStrDevicePath = (String *) pArgList->getDataFromIndex(1)) == NULL) {
			Debug::outError("GizmoPluginCPUUsage :: Invalid Call to getPlugin -- No DevicePath!");
			return NULL;
		}
		
		return new GizmoPluginCPUUsage(pScriptEngine, *pStrDevicePath);
	}
}

#endif // __GizmoPluginCPUUsage_h

