#if HAVE_CONFIG_H
#include "config.h"
#endif

#include <string.h>
#include <gtk/gtk.h>
#include <xmms/plugin.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <sys/types.h>
#include <stdio.h>

extern "C" {

// Func Prots
static void GizmoXMMS_init(void);
static void GizmoXMMS_cleanup(void);
static void GizmoXMMS_about(void);
static void GizmoXMMS_configure(void);
static void GizmoXMMS_playback_start(void);
static void GizmoXMMS_playback_stop(void);
static void GizmoXMMS_render_freq(gint16 freq_data[2][256]);

#define DEV_NAME	"XMMS Visualization"
#define DEV_NAME_ALT	"Song Name Here"
#define SHARED_MEM_ID	303
#define SHARED_MEM_SIZE	sizeof(unsigned char) * 5

// Global Vars
static unsigned char	m_CharData[256];
static unsigned char *	m_Data;
static key_t		m_SHMkey;
static int		m_SHMid;
//static shmid_ds 	m_ShmDS;
static unsigned char 	m_WriteNum = 0;
	
// Callback functions
VisPlugin GizmoXMMS_vtable = {
  NULL,					// Handle, filled in by XMMS
  NULL,					// Filename, filled in by XMMS
  0,					// Session ID
  NULL,					// description
  0, 					// # of PCM channels for render_pcm()
  1,					// # of freq channels wanted for render_freq()
  GizmoXMMS_init,           		// Called when plugin is enabled
  GizmoXMMS_cleanup,        		// Called when plugin is disabled
  GizmoXMMS_about,          		// Show the about box
  GizmoXMMS_configure,      		// Show the configure box
  NULL,                        		// Called to disable plugin, filled in by XMMS
  GizmoXMMS_playback_start, 		// Called when playback starts
  GizmoXMMS_playback_stop,  		// Called when playback stops
  NULL,                        		// Render the PCM data, must return quickly
  GizmoXMMS_render_freq     		// Render the freq data, must return quickly
};

// XMMS entry point
VisPlugin * get_vplugin_info(void) {
	GizmoXMMS_vtable.description = g_strdup_printf("GizmoDaemon VisPlugin XMMS -=- v%s", VERSION);

	return &GizmoXMMS_vtable;
}

} // EXTERN <-- important yo!

static void sendVisMessage(unsigned char Value0, unsigned char Value1) {
	if (!m_Data)
		return;
		
	m_Data[0] = m_WriteNum ++;
	m_Data[1] = Value0;
	m_Data[2] = Value1;
	m_Data[3] = (((int)Value0 + (int)Value1) / 2);
	m_Data[4] = 0;
}

static void GizmoXMMS_cleanup(void) {
	printf("GizmoXMMS plugin: Cleanup\n");
	
	if (m_Data) {
		m_Data[0] = ++ m_WriteNum;
		m_WriteNum ++;
		m_Data[4] = 2;
		shmdt((void *) m_Data);
		m_Data = NULL;
	}
	//if (m_HardClose)
	//	shmctl(m_SHMid, IPC_RMID, NULL);
	m_SHMid = -1;
	m_SHMkey = 0;
}

static void GizmoXMMS_init(void) {
	printf("GizmoXMMS plugin: Initializing\n");

	m_SHMid = shmget(SHARED_MEM_ID, SHARED_MEM_SIZE, 0600 | IPC_CREAT);
	m_Data = (unsigned char *) shmat(m_SHMid, NULL, 0);
	if (m_Data == (unsigned char *)(-1)) {
		GizmoXMMS_cleanup();
		return;
	}

	memset(m_Data, 0, SHARED_MEM_SIZE);
	m_Data[0] = ++ m_WriteNum;
	m_WriteNum ++;
	m_Data[4] = 1;
}

static void GizmoXMMS_about(void) {
	printf("GizmoXMMS plugin: About\n");
}

static void GizmoXMMS_configure(void) {
	printf("GizmoXMMS plugin: Configure\n");
}

static void GizmoXMMS_playback_start(void) {
	printf("GizmoXMMS plugin: Playback starting\n");

	sendVisMessage(255, 255);
}

static void GizmoXMMS_playback_stop(void) {
	printf("GizmoXMMS plugin: Playback stopping\n");

	sendVisMessage(0, 0);
}

static void GizmoXMMS_render_freq(gint16 freq_data[2][256]) {
	// convert that data yo!
	unsigned char highest0 = 0;
	//unsigned char highest1 = 0;
	
	for (int lp = 0; lp < 256; lp ++) {
		m_CharData[lp] = (unsigned char) (255.0 * double(freq_data[0][lp]) / 32768.0);
		if (m_CharData[lp] > highest0)
			highest0 = m_CharData[lp];
	}

	/*
	for (int lp = 0; lp < 256; lp ++) {
		m_CharData[lp] = (unsigned char) (255.0 * double(freq_data[1][lp]) / 32768.0);
		if (m_CharData[lp] > highest1)
			highest1 = m_CharData[lp];
	}
	*/
	
	sendVisMessage(highest0, highest0);
}
