  //////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
//
// GizmoPluginGeneric.cpp
//
//	GizmoPluginGeneric class
//
///////////////////////////////////////
  ///////////////////////////////////////
  
/*

Copyright (c) 2005, Tim Burrell
All rights reserved.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#include "GizmoPluginGeneric.hpp"
#include "libGizmo/Debug.hpp"
#include "libGizmo/UtilTime.hpp"
#include <linux/input.h>
#include <string.h>
#include <errno.h>
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdlib.h>

////////////////////////////////////////////////////////////////////////////
// Statics / Type Defs
///////////////////////////////////////

#define DEVICE_OPENMODE		O_RDWR
#define NUM_EVENT_DEVICES	16
#define IBUF_SIZE		32

////////////////////////////////////////////////////////////////////////////
// Overrides
///////////////////////////////////////

void GizmoPluginGeneric::onInputRelative(int Code, int Value) {
	//Debug::outDebug("GizmoPluginGeneric[%s] :: onGenericRelative: [%d] [%d]", m_DevicePath.getString(), Code, Value);

	if (!checkKeyRateLimit(Code, Value))
		return;
	
	const int IN_ARGS = 5;
	ScriptFuncArg * pInArg[IN_ARGS];	
	for (int lp = 0; lp < IN_ARGS; lp ++) {
		pInArg[lp] = m_ScriptFuncOnGenericRelative.getInArg(lp);
		if (!pInArg[lp]) {
			Debug::outDebug("GizmoPluginGeneric :: Failed to Call OnGenericRelative -- Arg %d", lp);
			return;
		}
	}
		
	pInArg[0]->ArgNumber = m_DeviceID;
	pInArg[1]->ArgNumber = Code;
	pInArg[2]->ArgNumber = Value;
	pInArg[3]->ArgString = m_DeviceName;
	pInArg[4]->ArgString = m_DevicePath;
	
	if (m_pScriptEngine)
		m_pScriptEngine->callScriptFunction(&m_ScriptFuncOnGenericRelative);
}

void GizmoPluginGeneric::onInputKey(int Code, int Value) {
	const int IN_ARGS = 5;
	
	//Debug::outDebug("GizmoPluginGeneric :: onInputKey [%d] [%d]", Code, Value);

	if (!checkKeyRateLimit(Code, Value))
		return;
	if (m_pScriptEngine == NULL)
		return;

	ScriptFuncArg * pInArg[IN_ARGS];	
	int lp;
	
	switch (Value) {
	case 0:
		// release
		for (lp = 0; lp < IN_ARGS; lp ++) {
			pInArg[lp] = m_ScriptFuncOnGenericRelease.getInArg(lp);
			if (!pInArg[lp]) {
				Debug::outDebug("GizmoPluginGeneric :: Failed to Call OnGenericRelease -- Arg %d", lp);
				return;
			}
		}
		
		pInArg[0]->ArgNumber = m_DeviceID;
		pInArg[1]->ArgNumber = Code;
		pInArg[2]->ArgString = getKeyStringFromCode(Code);
		pInArg[3]->ArgString = m_DeviceName;
		pInArg[4]->ArgString = m_DevicePath;
		
		m_pScriptEngine->callScriptFunction(&m_ScriptFuncOnGenericRelease);
		
		// now do for click
		for (lp = 0; lp < IN_ARGS; lp ++) {
			pInArg[lp] = m_ScriptFuncOnGenericClick.getInArg(lp);
			if (!pInArg[lp]) {
				Debug::outDebug("GizmoPluginGeneric :: Failed to Call OnGenericClick -- Arg %d", lp);
				return;
			}
		}
		
		pInArg[0]->ArgNumber = m_DeviceID;
		pInArg[1]->ArgNumber = Code;
		pInArg[2]->ArgString = getKeyStringFromCode(Code);
		pInArg[3]->ArgString = m_DeviceName;
		pInArg[4]->ArgString = m_DevicePath;
		
		m_pScriptEngine->callScriptFunction(&m_ScriptFuncOnGenericClick);
		break;
	case 1:
		// press
		for (lp = 0; lp < IN_ARGS; lp ++) {
			pInArg[lp] = m_ScriptFuncOnGenericPress.getInArg(lp);
			if (!pInArg[lp]) {
				Debug::outDebug("GizmoPluginGeneric :: Failed to Call OnGenericPress -- Arg %d", lp);
				return;
			}
		}
		
		pInArg[0]->ArgNumber = m_DeviceID;
		pInArg[1]->ArgNumber = Code;
		pInArg[2]->ArgString = getKeyStringFromCode(Code);
		pInArg[3]->ArgString = m_DeviceName;
		pInArg[4]->ArgString = m_DevicePath;
		
		m_pScriptEngine->callScriptFunction(&m_ScriptFuncOnGenericPress);
		break;
	case 2:
		Debug::outDebug("GizmoPluginGeneric :: onInputRepeat [%d] [%d]", Code, Value);
		break;
	}
}

void GizmoPluginGeneric::onInputAbsolute(int Code, int Value) {
	Debug::outDebug("GizmoPluginGeneric :: onInputAbsolute [%d] [%d]", Code, Value);

	if (!checkKeyRateLimit(Code, Value))
		return;
	
	const int IN_ARGS = 5;
	ScriptFuncArg * pInArg[IN_ARGS];	
	for (int lp = 0; lp < IN_ARGS; lp ++) {
		pInArg[lp] = m_ScriptFuncOnGenericAbsolute.getInArg(lp);
		if (!pInArg[lp]) {
			Debug::outDebug("GizmoPluginGeneric :: Failed to Call OnGenericAbsolute -- Arg %d", lp);
			return;
		}
	}
		
	pInArg[0]->ArgNumber = m_DeviceID;
	pInArg[1]->ArgNumber = Code;
	pInArg[2]->ArgNumber = Value;
	pInArg[3]->ArgString = m_DeviceName;
	pInArg[4]->ArgString = m_DevicePath;
	
	if (m_pScriptEngine)
		m_pScriptEngine->callScriptFunction(&m_ScriptFuncOnGenericAbsolute);
}

void GizmoPluginGeneric::onInputMisc(int Code, int Value) {
	//Debug::outDebug("GizmoPluginGeneric :: onInputMisc [%d] [%d]", Code, Value);

	if (!checkKeyRateLimit(Code, Value))
		return;
	
	const int IN_ARGS = 5;
	ScriptFuncArg * pInArg[IN_ARGS];	
	for (int lp = 0; lp < IN_ARGS; lp ++) {
		pInArg[lp] = m_ScriptFuncOnGenericMisc.getInArg(lp);
		if (!pInArg[lp]) {
			Debug::outDebug("GizmoPluginGeneric :: Failed to Call OnGenericMisc -- Arg %d", lp);
			return;
		}
	}
		
	pInArg[0]->ArgNumber = m_DeviceID;
	pInArg[1]->ArgNumber = Code;
	pInArg[2]->ArgNumber = Value;
	pInArg[3]->ArgString = m_DeviceName;
	pInArg[4]->ArgString = m_DevicePath;
	
	if (m_pScriptEngine)
		m_pScriptEngine->callScriptFunction(&m_ScriptFuncOnGenericMisc);
}

void GizmoPluginGeneric::onInputLED(int Code, int Value) {
	Debug::outDebug("GizmoPluginGeneric :: onInputLED [%d] [%d]", Code, Value);
}

void GizmoPluginGeneric::onInputSyn(int Code, int Value) {
	//Debug::outDebug("GizmoPluginGeneric :: onInputSYN [%d] [%d]", Code, Value);
}

bool GizmoPluginGeneric::onPluginOpened() {
	Debug::outDebug("GizmoPluginGeneric :: onPluginOpened");
	
	return initPlugin();
}

////////////////////////////////////////////////////////////////////////////
// Construction
///////////////////////////////////////

GizmoPluginGeneric::GizmoPluginGeneric(ScriptEngineInterface * pScriptEngine, const char * DevicePath) : 
	GizmoDeviceInput(DevicePath),
	m_ScriptFuncGenericSetGrabExclusiveAccess(this),
	m_ScriptFuncGenericSendKeyEventString(this),
	m_ScriptFuncGenericSendKeyEvent(this),
	m_ScriptFuncGenericSendRelativeEvent(this),
	m_ScriptFuncGenericSetKeyRateLimit(this)
{
	Debug::outDebug("GizmoPluginGeneric :: Generic Input Plugin Loading [%s]...", DevicePath);
	
	m_pScriptEngine = pScriptEngine;
	m_DevicePath = DevicePath;
	m_KeyRateLimit = 0;
	m_LastKey = 0;
	m_LastCode = -1;
	m_LastValue = -1;
	m_Blocking = 0;
}

GizmoPluginGeneric::~GizmoPluginGeneric() {
	closePlugin();
	Debug::outDebug("GizmoPluginGeneric :: Generic Input Plugin Unloaded.");
}

////////////////////////////////////////////////////////////////////////////
// Class Body
///////////////////////////////////////

void GizmoPluginGeneric::closePlugin() {
	Debug::outDebug("GizmoPluginGeneric :: Generic Input Plugin Closing...");
}

bool GizmoPluginGeneric::initPlugin() {
	Debug::outDebug("    Generic Input Plugin v%s [%s]", PACKAGE_VERSION, m_DevicePath.getString());

	if (!m_pScriptEngine) {
		Debug::outError("GizmoPluginGeneric :: Invalid Script Engine!");
		return false;
	}

	// try to open the device
	if (!initDevice()) {
		Debug::outDebug("GizmoPluginGeneric :: Generic Input Plugin Failed to Initialize!");
		return false;
	}
		
	// register script functions
	Debug::outDebug("GizmoPluginGeneric :: Registering Functions...");
	m_pScriptEngine->registerScriptFunction(&m_ScriptFuncGenericSetGrabExclusiveAccess);
	m_pScriptEngine->registerScriptFunction(&m_ScriptFuncGenericSendKeyEventString);
	m_pScriptEngine->registerScriptFunction(&m_ScriptFuncGenericSendKeyEvent);
	m_pScriptEngine->registerScriptFunction(&m_ScriptFuncGenericSendRelativeEvent);
	m_pScriptEngine->registerScriptFunction(&m_ScriptFuncGenericSetKeyRateLimit);

	Debug::out("\t- Found [%s] at [%s]", m_DeviceName.getString(), m_DevicePath.getString());	
	Debug::outDebug("GizmoPluginGeneric :: Generic Input Plugin Initialized.");
	return true;
}

String GizmoPluginGeneric::getDeviceName() {
	return m_DeviceName;
}

String GizmoPluginGeneric::getDeviceNameAlt() {
	return m_DevicePath;
}

void GizmoPluginGeneric::scriptFunction_RemapKey(ScriptFunction * pFunc) {
	if (!ScriptFunction::checkScriptFunc(pFunc))
		return;

	GizmoPluginGeneric * pThis = (GizmoPluginGeneric *) pFunc->Arguments;
	
	int CurCode = 0, NewCode = 0;
	
	// get arguments
	ScriptFuncArg * pArg1 = NULL;
	ScriptFuncArg * pArg2 = NULL;
	if (pFunc->InArgList.getSize() >= 2) {
		pArg1 = pFunc->InArgList.getDataFromIndex(0);
		pArg2 = pFunc->InArgList.getDataFromIndex(1);
		if ( (!pArg1) || (!pArg2) )
			return;
		if (pArg1->Type == SCRIPT_ARG_NUMBER)
			CurCode = (int) pArg1->ArgNumber;
		else if (pArg1->Type == SCRIPT_ARG_STRING)
			CurCode = pThis->getKeyCodeFromString(pArg1->ArgString);
		else
			return;
		if (pArg2->Type == SCRIPT_ARG_NUMBER)
			NewCode = (int) pArg2->ArgNumber;
		else if (pArg1->Type == SCRIPT_ARG_STRING)
			NewCode = pThis->getKeyCodeFromString(pArg2->ArgString);
		else
			return;
	} else
		return;

	// set key event
	bool ret = pThis->remapKey(CurCode, NewCode);
	if (ret)
		Debug::out("\t- Key Remapped from [%d] to [%d]!", CurCode, NewCode);
	else
		Debug::out("\t- Failed to Remap Key from [%d] to [%d]!", CurCode, NewCode);
	    
	// return true
	pArg1 = new ScriptFuncArg();
	pArg1->Type = SCRIPT_ARG_BOOLEAN;
	pArg1->ArgBoolean = ret;
	pFunc->OutArgList.addTail(pArg1);
}

bool GizmoPluginGeneric::sendRelEvent(int Code, int Value) {
	bool ret1 = writeToDevice(EV_REL, Code, Value);
	bool ret2 = writeToDevice(EV_SYN, 0, 0);
	if (ret1 && ret2) {
		Debug::outDebug("GizmoPluginGeneric :: Wrote to Device");
		return true;
	} else {
		Debug::outDebug("GizmoPluginGeneric :: Failed to Write to Device");
		return false;
	}
}

void GizmoPluginGeneric::setKeyRateLimit(int Limit) {
	if (Limit < 0)
		return;
		
	// set key event
	m_KeyRateLimit = (int) Limit * 1000;
	m_LastKey = UtilTime::getTicks();

	Debug::out("        - Set Max KeyRate Limit to [%d] microseconds", m_KeyRateLimit);
}

bool GizmoPluginGeneric::checkKeyRateLimit(int Code, int Value) {
	if (m_KeyRateLimit == 0)
		return true;

	m_Blocking ++;

	unsigned long curTime = UtilTime::getTicks();
	bool RealUnblock = ((curTime - m_LastKey > m_KeyRateLimit) || (Code != m_LastCode));
	if (RealUnblock || (m_Blocking <= 2) ) {
		m_LastKey = UtilTime::getTicks();
		m_LastCode = Code;
		m_LastValue = Value;
		if (RealUnblock) {
			m_Blocking = 0;
		}
		return true;
	}

	//Debug::out("**LIMIT!!");

	return false;
}

bool GizmoPluginGeneric::setGrabExclusiveAccess(bool DoGrab) {
	// set key event
	bool ret = grabExclusiveAccess(DoGrab);
	if (ret)
		Debug::out("\t- Exclusive Access Granted");
	else
		Debug::out("\t- Exclusive Access Failed!");
		
	return ret;	
}

bool GizmoPluginGeneric::sendKeyEvent(int Code, int Value) {
	bool ret1 = writeToDevice(EV_KEY, Code, Value);
	bool ret2 = writeToDevice(EV_SYN, 0, 0);
	if (ret1 && ret2)
		Debug::outDebug("GizmoPluginGeneric :: Wrote to Device");
	else {
		Debug::outDebug("GizmoPluginGeneric :: Failed to Write to Device");
		return false;
	}
	    
	return true;
}

bool GizmoPluginGeneric::sendKeyEventString(String Code, int Value) {
	bool ret1 = writeToDevice(EV_KEY, getKeyCodeFromString(Code), Value);
	bool ret2 = writeToDevice(EV_SYN, 0, 0);
	if (ret1 && ret2)
		Debug::outDebug("GizmoPluginGeneric :: Wrote to Device");
	else {
		Debug::outDebug("GizmoPluginGeneric :: Failed to Write to Device");
		return false;
	}
	    
	// return true
	return true;
}

bool GizmoPluginGeneric::checkEventsWaiting() {
	return GizmoDeviceInput::checkEventsWaiting();
}
