  ///////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////
//
// GizmoPluginGeneric.hpp
//
//	GizmoPluginGeneric header file
//
///////////////////////////////////////
  ///////////////////////////////////////
  
/*

Copyright (c) 2006, Tim Burrell
All rights reserved.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#ifndef __GizmoPluginGeneric_h
#define __GizmoPluginGeneric_h

#if HAVE_CONFIG_H
#include "config.h"
#endif

#include "libGizmo/ScriptEngineInterface.hpp"
#include "libGizmo/String.hpp"
#include "libGizmo/GizmoDeviceInput.hpp"
#include "libGizmo/Debug.hpp"
#include "GenericInterface.hpp"
#include "ScriptFunctionOnGenericPress.hpp"
#include "ScriptFunctionOnGenericRelease.hpp"
#include "ScriptFunctionOnGenericClick.hpp"
#include "ScriptFunctionGenericSetGrabExclusiveAccess.hpp"
#include "ScriptFunctionGenericSendKeyEventString.hpp"
#include "ScriptFunctionGenericSendKeyEvent.hpp"
#include "ScriptFunctionGenericSendRelativeEvent.hpp"
#include "ScriptFunctionGenericSetKeyRateLimit.hpp"
#include "ScriptFunctionOnGenericRelative.hpp"
#include "ScriptFunctionOnGenericAbsolute.hpp"
#include "ScriptFunctionOnGenericMisc.hpp"

//////////////////////////////////////////////////////////////////////////////
// Typedefs
///////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
// Class Definition
///////////////////////////////////////

class GizmoPluginGeneric : public GenericInterface, public GizmoDeviceInput {
public:
	bool		checkEventsWaiting();
	void 		closePlugin();
	String		getDeviceName();
	String		getDeviceNameAlt();
	bool		initPlugin();
	virtual void	onInputAbsolute(int Code, int Value);
	virtual void	onInputKey(int Code, int Value);
	virtual void	onInputLED(int Code, int Value);
	virtual void	onInputMisc(int Code, int Value);
	virtual void	onInputRelative(int Code, int Value);
	virtual void	onInputSyn(int Code, int Value);
	virtual bool	onPluginOpened();
	virtual bool	setGrabExclusiveAccess(bool DoGrab);
	virtual bool	sendKeyEvent(int Code, int Value);
	virtual bool	sendKeyEventString(String Code, int Value);
	virtual bool	sendRelEvent(int Code, int Value);
	virtual void	setKeyRateLimit(int Limit);
	
	GizmoPluginGeneric(ScriptEngineInterface * pScriptEngine, const char * DevicePath);
	virtual ~GizmoPluginGeneric();
	
	static void 	scriptFunction_RemapKey(ScriptFunction * pFunc);
	static void 	scriptFunction_SendRelEvent(ScriptFunction * pFunc);

private:
	bool 		checkKeyRateLimit(int Code, int Value);
	
	ScriptEngineInterface * m_pScriptEngine;
	unsigned long	m_KeyRateLimit;
	unsigned long	m_LastKey;
	int		m_LastCode;
	int		m_LastValue;
	int		m_Blocking;
	
	ScriptFunctionOnGenericPress 			m_ScriptFuncOnGenericPress;
	ScriptFunctionOnGenericRelease 			m_ScriptFuncOnGenericRelease;
	ScriptFunctionOnGenericClick 			m_ScriptFuncOnGenericClick;
	ScriptFunctionOnGenericRelative			m_ScriptFuncOnGenericRelative;
	ScriptFunctionOnGenericAbsolute			m_ScriptFuncOnGenericAbsolute;
	ScriptFunctionOnGenericMisc			m_ScriptFuncOnGenericMisc;
	
	ScriptFunctionGenericSetGrabExclusiveAccess	m_ScriptFuncGenericSetGrabExclusiveAccess;
	ScriptFunctionGenericSendKeyEventString		m_ScriptFuncGenericSendKeyEventString;
	ScriptFunctionGenericSendKeyEvent		m_ScriptFuncGenericSendKeyEvent;
	ScriptFunctionGenericSendRelativeEvent		m_ScriptFuncGenericSendRelativeEvent;
	ScriptFunctionGenericSetKeyRateLimit		m_ScriptFuncGenericSetKeyRateLimit;
};

//////////////////////////////////////////////////////////////////////////////
// Shared Object Attach Function
///////////////////////////////////////

extern "C" {
	GizmoPlugin * getPlugin(LinkList<void> * pArgList) {
		ScriptEngineInterface * pScriptEngine;
		if ((pScriptEngine = (ScriptEngineInterface *) pArgList->getDataFromIndex(0)) == NULL) {
			Debug::outError("GizmoPluginGeneric :: Invalid Call to getPlugin -- No ScriptEngine!");
			return NULL;
		}
		
		String * pStrDevicePath;
		if ((pStrDevicePath = (String *) pArgList->getDataFromIndex(1)) == NULL) {
			Debug::outError("GizmoPluginGeneric :: Invalid Call to getPlugin -- No DevicePath!");
			return NULL;
		}
		
		return new GizmoPluginGeneric(pScriptEngine, *pStrDevicePath);
	}
}

#endif // __GizmoPluginGeneric_h
