  //////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
//
// ScriptFunctionGenericSendKeyEventString.cpp
//
//	ScriptFunctionGenericSendKeyEventString class
//
///////////////////////////////////////
  ///////////////////////////////////////
  
/*

Copyright (c) 2005, Tim Burrell
All rights reserved.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#include "ScriptFunctionGenericSendKeyEventString.hpp"
#include "libGizmo/Debug.hpp"

////////////////////////////////////////////////////////////////////////////
// Statics / Type Defs
///////////////////////////////////////

////////////////////////////////////////////////////////////////////////////
// Overrides
///////////////////////////////////////

bool ScriptFunctionGenericSendKeyEventString::executeFunction() {
	//Debug::outDebug("ScriptFunctionGenericSendKeyEventString :: executeFunction");
	
	ScriptFuncArg * pOutArg = getInArg(0);
	if (!pOutArg) {
		Debug::out("ScriptFunctionGenericSendKeyEventString :: executeFunction -- No Out Parameter!");
		return false;
	}
	
	ScriptFuncArg * pInArg1 = getInArg(0);
	if (!pInArg1) {
		Debug::out("ScriptFunctionGenericSendKeyEventString :: executeFunction -- No In Parameter 1!");
		return false;
	}
	
	ScriptFuncArg * pInArg2 = getInArg(1);
	if (!pInArg2) {
		Debug::out("ScriptFunctionGenericSendKeyEventString :: executeFunction -- No Out Parameter 2!");
		return false;
	}
	
	ScriptFuncArg * pInArg3 = getInArg(2);
	if (!pInArg3) {
		Debug::out("ScriptFunctionGenericSendKeyEventString :: executeFunction -- No Out Parameter 3!");
		return false;
	}
	
	// find device
	LinkList<GenericInterface> * pPluginList = GenericInterface::getPluginList();
	if (pPluginList->getSize() == 0) {
		Debug::out("ScriptFunctionGenericSendKeyEventString :: PluginList Badness!");
		return false;
	}
	pPluginList->moveToHead();
	do {
		GenericInterface * pInt = pPluginList->getData();
		if (pInt->getDeviceID() == pInArg1->ArgNumber) {
			pOutArg->ArgBoolean = pInt->sendKeyEventString(pInArg2->ArgString, (int) pInArg3->ArgNumber);
			break;
		}
	} while (pPluginList->moveNext());
		
	return true;
}

////////////////////////////////////////////////////////////////////////////
// Global Funcs
///////////////////////////////////////

////////////////////////////////////////////////////////////////////////////
// Construction
///////////////////////////////////////

ScriptFunctionGenericSendKeyEventString::ScriptFunctionGenericSendKeyEventString(GenericInterface * pPlugin) {
	m_pPlugin = pPlugin;
	FunctionName.format("genericSendKeyEventString");
	FunctionDescription = "Sends a Key Code to a Device via String";
	
	addInArg(SCRIPT_ARG_NUMBER);		// IN  DeviceID
	addInArg(SCRIPT_ARG_STRING);		// IN  Key String
	addInArg(SCRIPT_ARG_NUMBER);		// IN  Value (1) on (0) for off
	addOutArg(SCRIPT_ARG_BOOLEAN);		// OUT Success
}

ScriptFunctionGenericSendKeyEventString::~ScriptFunctionGenericSendKeyEventString() {
}

////////////////////////////////////////////////////////////////////////////
// Class Body
///////////////////////////////////////
