  //////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
//
// GizmoPluginLIRC.cpp
//
//	GizmoPluginLIRC class
//
///////////////////////////////////////
  ///////////////////////////////////////
  
/*

Copyright (c) 2005, Tim Burrell
All rights reserved.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#include "GizmoPluginLIRC.hpp"
#include "libGizmo/Debug.hpp"
#include "libGizmo/UtilTime.hpp"
#include <linux/input.h>
#include <string.h>
#include <errno.h>
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdlib.h>

////////////////////////////////////////////////////////////////////////////
// Statics / Type Defs
///////////////////////////////////////

////////////////////////////////////////////////////////////////////////////
// Overrides
///////////////////////////////////////

void GizmoPluginLIRC::onLIRCEvent(String Code, int Value, String Button, String DeviceName) {
	//Debug::outDebug("GizmoPluginLIRC :: LIRC Event [%s] [%s] <%d>", Button.getString(), DeviceName.getString(), Value);

	if (!checkKeyRateLimit(Value))
		return;
	
	const int IN_ARGS = 6;
	ScriptFuncArg * pInArg[IN_ARGS];	
	for (int lp = 0; lp < IN_ARGS; lp ++) {
		pInArg[lp] = m_ScriptFuncOnLIRCClick.getInArg(lp);
		if (!pInArg[lp]) {
			Debug::outDebug("GizmoPluginLIRC :: Failed to Call OnLIRCClick -- Arg %d", lp);
			return;
		}
	}
		
	pInArg[0]->ArgNumber = m_DeviceID;
	pInArg[1]->ArgString = Code;
	pInArg[2]->ArgNumber = Value;
	pInArg[3]->ArgString = Button;
	pInArg[4]->ArgString = DeviceName;
	pInArg[5]->ArgString = m_DevicePath;
	
	if (m_pScriptEngine)
		m_pScriptEngine->callScriptFunction(&m_ScriptFuncOnLIRCClick);
}

bool GizmoPluginLIRC::onPluginOpened() {
	Debug::outDebug("GizmoPluginLIRC :: onPluginOpened");
	
	return initPlugin();
}

////////////////////////////////////////////////////////////////////////////
// Construction
///////////////////////////////////////

GizmoPluginLIRC::GizmoPluginLIRC(ScriptEngineInterface * pScriptEngine, const char * DevicePath) : 
	m_ScriptFuncLIRCSetKeyRateLimit(this)
{
	Debug::outDebug("GizmoPluginLIRC :: LIRC Input Plugin Loading [%s]...", DevicePath);
	
	m_pScriptEngine = pScriptEngine;
	m_DeviceName = "GizmoPluginLIRC";
	m_DevicePath = DevicePath;
	m_KeyRateLimit = 0;
}

GizmoPluginLIRC::~GizmoPluginLIRC() {
	closePlugin();
	Debug::outDebug("GizmoPluginLIRC :: LIRC Input Plugin Unloaded.");
}

////////////////////////////////////////////////////////////////////////////
// Class Body
///////////////////////////////////////

void GizmoPluginLIRC::closePlugin() {
	Debug::outDebug("GizmoPluginLIRC :: LIRC Input Plugin Closing...");
	
	closeLIRC();
}

bool GizmoPluginLIRC::initPlugin() {
	Debug::outDebug("    LIRC Input Plugin v%s [%s]", PACKAGE_VERSION, m_DevicePath.getString());

	if (!m_pScriptEngine) {
		Debug::outError("GizmoPluginLIRC :: Invalid Script Engine!");
		return false;
	}

	// try to open the device
	if (m_DevicePath == "auto")
		m_DevicePath = "/dev/lircd";
	if (!initLIRC(m_DevicePath)) {
		Debug::outDebug("GizmoPluginLIRC :: LIRC Input Plugin Failed to Initialize!");
		return false;
	}
		
	// register script functions
	Debug::outDebug("GizmoPluginLIRC :: Registering Functions...");
	m_pScriptEngine->registerScriptFunction(&m_ScriptFuncLIRCSetKeyRateLimit);

	Debug::out("\t- Found [%s] at [%s]", m_DeviceName.getString(), m_DevicePath.getString());	
	Debug::outDebug("GizmoPluginLIRC :: LIRC Input Plugin Initialized.");
	return true;
}

String GizmoPluginLIRC::getDeviceName() {
	return m_DeviceName;
}

String GizmoPluginLIRC::getDeviceNameAlt() {
	return m_DevicePath;
}

void GizmoPluginLIRC::setKeyRateLimit(int Limit) {
	if (Limit < 0)
		return;
		
	// set key event
	m_KeyRateLimit = Limit;

	Debug::out("        - Set Max KeyRate Limit to [%d]", m_KeyRateLimit);
}

bool GizmoPluginLIRC::checkKeyRateLimit(int Value) {
	if ( (m_KeyRateLimit == 0) || (Value <= 0) )
		return true;

	if ((unsigned) Value <= m_KeyRateLimit)
		return false;

	return true;
}

bool GizmoPluginLIRC::checkEventsWaiting() {
	return LIRC::checkEventsWaiting();
}
