  ///////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////
//
// GizmoPluginLIRC.hpp
//
//	GizmoPluginLIRC header file
//
///////////////////////////////////////
  ///////////////////////////////////////
  
/*

Copyright (c) 2006, Tim Burrell
All rights reserved.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#ifndef __GizmoPluginLIRC_h
#define __GizmoPluginLIRC_h

#if HAVE_CONFIG_H
#include "config.h"
#endif

#include "libGizmo/ScriptEngineInterface.hpp"
#include "libGizmo/String.hpp"
#include "libGizmo/Debug.hpp"
#include "LIRCInterface.hpp"
#include "LIRC.hpp"
#include "ScriptFunctionOnLIRCClick.hpp"
#include "ScriptFunctionLIRCSetKeyRateLimit.hpp"

//////////////////////////////////////////////////////////////////////////////
// Typedefs
///////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
// Class Definition
///////////////////////////////////////

class GizmoPluginLIRC : public LIRCInterface, public LIRC {
public:
	bool		checkEventsWaiting();
	void 		closePlugin();
	String		getDeviceName();
	String		getDeviceNameAlt();
	bool		initPlugin();
	virtual void	onLIRCEvent(String Code, int Value, String Button, String DeviceName);
	virtual bool	onPluginOpened();
	virtual void	setKeyRateLimit(int Limit);
	
	GizmoPluginLIRC(ScriptEngineInterface * pScriptEngine, const char * DevicePath);
	virtual ~GizmoPluginLIRC();
	
private:
	bool 		checkKeyRateLimit(int Value);
	
	ScriptEngineInterface * m_pScriptEngine;
	unsigned long	m_KeyRateLimit;
	
	ScriptFunctionOnLIRCClick 			m_ScriptFuncOnLIRCClick;
	ScriptFunctionLIRCSetKeyRateLimit		m_ScriptFuncLIRCSetKeyRateLimit;
};

//////////////////////////////////////////////////////////////////////////////
// Shared Object Attach Function
///////////////////////////////////////

extern "C" {
	GizmoPlugin * getPlugin(LinkList<void> * pArgList) {
		ScriptEngineInterface * pScriptEngine;
		if ((pScriptEngine = (ScriptEngineInterface *) pArgList->getDataFromIndex(0)) == NULL) {
			Debug::outError("GizmoPluginLIRC :: Invalid Call to getPlugin -- No ScriptEngine!");
			return NULL;
		}
		
		String * pStrDevicePath;
		if ((pStrDevicePath = (String *) pArgList->getDataFromIndex(1)) == NULL) {
			Debug::outError("GizmoPluginLIRC :: Invalid Call to getPlugin -- No DevicePath!");
			return NULL;
		}
		
		return new GizmoPluginLIRC(pScriptEngine, *pStrDevicePath);
	}
}

#endif // __GizmoPluginLIRC_h
