  //////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
//
// LIRC.cpp
//
//	LIRC class
//
///////////////////////////////////////
  ///////////////////////////////////////
  
/*

Copyright (c) 2005, Tim Burrell
All rights reserved.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#include "LIRC.hpp"
#include "libGizmo/Debug.hpp"
#include <errno.h>
#include <stdlib.h>

////////////////////////////////////////////////////////////////////////////
// Statics / Type Defs
///////////////////////////////////////

#define LIRCD	"/dev/lircd"

////////////////////////////////////////////////////////////////////////////
// Overrides
///////////////////////////////////////

////////////////////////////////////////////////////////////////////////////
// Construction
///////////////////////////////////////

LIRC::LIRC() {
	m_DevPath = LIRCD;
	m_FD = -1;
}

LIRC::~LIRC() {
	closeLIRC();
}

////////////////////////////////////////////////////////////////////////////
// Class Body
///////////////////////////////////////

bool LIRC::checkEventsWaiting(){
	int 		r;
	//int		events, i;
	fd_set 		readfds;
	struct timeval	TimeOut;
	bool		ret = false;

	TimeOut.tv_sec = 0;
	TimeOut.tv_usec = 100000;
		
	// tell select which sockets to poll
	FD_ZERO(&readfds);
	FD_SET(m_FD, &readfds);

	// wait on sockets
	int NumSelected = select(m_FD + 1, &readfds, NULL, NULL, &TimeOut);
	if (NumSelected == -1)
		return false;
	else if (NumSelected == 0) // timeout
		return false;
		
	if (FD_ISSET(m_FD, &readfds)) {
		ret = true;
		// read event!
		r = read(m_FD, m_Buf, sizeof(char) * LIRC_BUF_SIZE);
		if (r > 0) {
			processEvent();
		} else {
			Debug::outError("LIRC :: Failed to Read from Device!: %s", strerror(errno));
		}
	}
	
	return ret;
}

/*
void * LIRC::threadFunction() {
	m_Watching = true;
	while (m_Watching) {
		int i = read(m_FD, m_Buf, LIRC_BUF_SIZE);
		if (i == -1) {
			Debug::outError("LIRC :: Failed to Read from LIRCD!");
			m_Watching = false;
			return NULL;
		}
		
		if (!i)
			continue;
			
		Debug::outDebug("LIRC :: Received [%s]", m_Buf);
	};
	
	return NULL;
}
*/

bool LIRC::initLIRC(String DevPath) {
	if (DevPath != "")
		m_DevPath = DevPath;
		
	m_Addr.sun_family = AF_UNIX;
	strcpy(m_Addr.sun_path, m_DevPath.getString());
	
	if ((m_FD = socket(AF_UNIX,SOCK_STREAM, 0)) == -1) {
		Debug::outError("LIRC :: Failed to Create Communication Socket!");
		return false;
	}
	
	if (connect(m_FD,(struct sockaddr *) &m_Addr,sizeof(m_Addr)) == -1)  {
		Debug::outError("LIRC :: Failed to Connect to LIRCD!");
		return false;
	}
	
	return true;
}

void LIRC::closeLIRC() {	
	if (m_FD != -1) {
		close(m_FD);
		m_FD = -1;
	}
	m_Watching = false;
}

bool LIRC::processEvent() {
	const int FIELDS = 4;
	
	String Buffer = m_Buf;
	Buffer.replaceSubstring("\n", " ");
	Buffer += " ";
	int sPos[FIELDS];
	String Field[FIELDS];
	int lPos = 0;
	int cPos = 0;
	int lp;
	for (lp = 0; lp < FIELDS; lp ++) {
		if ((sPos[lp] = Buffer.find(" ", cPos)) == -1) {
			Debug::outDebug("LIRC :: Invalid Code Received [%s]", Buffer.getString());
			return false;
		}
		lPos = cPos;
		cPos = sPos[lp] + 1;
		
		Field[lp] = Buffer.mid(lPos, cPos - 2);
		//Debug::out("Field[%d] == %s", lp, Field[lp].getString());
	}
	
	onLIRCEvent(Field[0], atoi(Field[1].getString()), Field[2], Field[3]);
	
	return true;
}
