  ///////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////
//
// GizmoPluginPowerMate.hpp
//
//	GizmoPluginPowerMate header file
//
///////////////////////////////////////
  ///////////////////////////////////////
  
/*

Copyright (c) 2005, Tim Burrell
All rights reserved.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#ifndef __GizmoPluginPowerMate_h
#define __GizmoPluginPowerMate_h

#if HAVE_CONFIG_H
#include "config.h"
#endif

#include "libGizmo/ScriptEngineInterface.hpp"
#include "libGizmo/String.hpp"
#include "libGizmo/GizmoDeviceInput.hpp"
#include "libGizmo/Debug.hpp"
#include "PowerMateInterface.hpp"
#include "ScriptFunctionOnPowerMateRotate.hpp"
#include "ScriptFunctionOnPowerMatePress.hpp"
#include "ScriptFunctionOnPowerMateClick.hpp"
#include "ScriptFunctionOnPowerMateRelease.hpp"
#include "ScriptFunctionOnPowerMateLEDChange.hpp"
#include "ScriptFunctionPowerMateGetLED.hpp"
#include "ScriptFunctionPowerMateGetLEDPercent.hpp"
#include "ScriptFunctionPowerMateSetLED.hpp"
#include "ScriptFunctionPowerMateSetLEDPercent.hpp"
#include "ScriptFunctionPowerMateSetRotateSensitivity.hpp"
#include "ScriptFunctionPowerMateSetSleepModePulsing.hpp"
#include "ScriptFunctionPowerMateBlinkLED.hpp"
#include "ScriptFunctionPowerMateSetPulsing.hpp"

//////////////////////////////////////////////////////////////////////////////
// Typedefs
///////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
// Class Definition
///////////////////////////////////////

class GizmoPluginPowerMate : public PowerMateInterface, public GizmoDeviceInput {
public:
	void		blinkLED();
	bool		checkEventsWaiting();
	void 		closePlugin();
	String		getDeviceName();
	String		getDeviceNameAlt();
	unsigned char 	getLEDLevel();
	bool		initPlugin();
	virtual void	onClosing();
	virtual void	onInputKey(int Code, int Value);
	virtual void	onInputMisc(int Code, int Value);
	virtual void	onInputRelative(int Code, int Value);
	virtual bool	onPluginOpened();
	virtual int	onPowerMateGetLED();
	virtual void 	onPowerMateSetLED(int Level);
	virtual void	onPowerMateSetRotateSensitivity(int Level);
	virtual void	onPowerMateSetSleepModePulsing(bool DoPulse);
	virtual void	onPowerMateSetPulsing(int Speed, int Table);
	void 		pulseLED(int pulseTable, int pulseSpeed, bool ResetBlink = true);
	void 		pulseLED(int static_brightness, int pulse_speed, int pulse_table, int pulse_asleep, int pulse_awake, bool ResetBlink = true);
	void 		setLED(unsigned char level, bool ResetBlink = true);
	void 		setLED(float percent, bool ResetBlink = true);

	GizmoPluginPowerMate(ScriptEngineInterface * pScriptEngine, const char * DevicePath);
	virtual ~GizmoPluginPowerMate();

private:
	bool 		findPowerMate(int number = -1);
	
	ScriptEngineInterface * m_pScriptEngine;
	bool		m_Turned;
	int		m_Button;
	int		m_Prefix;
	int		m_RotSensitivity;
	int		m_RotLast;
	int		m_RotCount;
	int		m_Level;
	int		m_LevelPreBlink;
	int		m_Blinking;
	int		m_BlinkWatching;
	int		m_PulseAsleep;
	bool		m_BlackOnClose;
	
	ScriptFunctionOnPowerMateRotate			m_ScriptFuncOnPowerMateRotate;
	ScriptFunctionOnPowerMatePress			m_ScriptFuncOnPowerMatePress;
	ScriptFunctionOnPowerMateRelease		m_ScriptFuncOnPowerMateRelease;
	ScriptFunctionOnPowerMateClick	 		m_ScriptFuncOnPowerMateClick;
	ScriptFunctionOnPowerMateLEDChange 		m_ScriptFuncOnPowerMateLEDChange;
	
	ScriptFunctionPowerMateGetLED			m_ScriptFuncPowerMateGetLED;
	ScriptFunctionPowerMateGetLEDPercent		m_ScriptFuncPowerMateGetLEDPercent;
	ScriptFunctionPowerMateSetLED			m_ScriptFuncPowerMateSetLED;
	ScriptFunctionPowerMateSetLEDPercent		m_ScriptFuncPowerMateSetLEDPercent;
	ScriptFunctionPowerMateSetRotateSensitivity	m_ScriptFuncPowerMateSetRotateSensitivity;
	ScriptFunctionPowerMateSetSleepModePulsing	m_ScriptFuncPowerMateSetSleepModePulsing;
	ScriptFunctionPowerMateBlinkLED			m_ScriptFuncPowerMateBlinkLED;
	ScriptFunctionPowerMateSetPulsing		m_ScriptFuncPowerMateSetPulsing;
	
	static int	m_TotalFound;
};

//////////////////////////////////////////////////////////////////////////////
// Shared Object Attach Function
///////////////////////////////////////

extern "C" {
	GizmoPlugin * getPlugin(LinkList<void> * pArgList) {
		ScriptEngineInterface * pScriptEngine;
		if ((pScriptEngine = (ScriptEngineInterface *) pArgList->getDataFromIndex(0)) == NULL) {
			Debug::outError("GizmoPluginPowerMate :: Invalid Call to getPlugin -- No ScriptEngine!");
			return NULL;
		}
		
		String * pStrDevicePath;
		if ((pStrDevicePath = (String *) pArgList->getDataFromIndex(1)) == NULL) {
			Debug::outError("GizmoPluginPowerMate :: Invalid Call to getPlugin -- No DevicePath!");
			return NULL;
		}
		
		return new GizmoPluginPowerMate(pScriptEngine, *pStrDevicePath);
	}
}

#endif // __GizmoPluginPowerMate_h

