  ///////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////
//
// ScriptEnginePython.hpp
//
//	ScriptEnginePython header file
//
///////////////////////////////////////
  ///////////////////////////////////////
  
/*

Copyright (c) 2005, Tim Burrell
All rights reserved.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#ifndef __ScriptEnginePython_h
#define __ScriptEnginePython_h

#include <Python.h>

#if HAVE_CONFIG_H
#include "config.h"
#endif

#include "libGizmo/ScriptEngineInterface.hpp"
#include "libGizmo/ScriptFunction.hpp"
#include "libGizmo/LinkList.hpp"
#include <pthread.h>

//////////////////////////////////////////////////////////////////////////////
// Typdefs, Structs, etc
///////////////////////////////////////

const int PYTHON_MAX_FUNCTIONS = 100;

//////////////////////////////////////////////////////////////////////////////
// Class Definition
///////////////////////////////////////

class ScriptEnginePython : public ScriptEngineInterface {
public:
	bool		callScriptFunction(ScriptFunction * pFunction, int RecursionDepth = 0);
	bool		checkEventsWaiting();
	void		closePlugin();
	bool		initPlugin();
	int		getDeviceID();
	String		getDeviceName();
	String		getDeviceNameAlt();
	bool		onPluginOpened();
	bool		registerScriptFunction(ScriptFunction * pFunction);
	int 		runScript(const char * FileName);

	ScriptEnginePython();
	virtual ~ScriptEnginePython();
	
private:
	bool 		buildPyMethods();
	bool		executeScript();
	void		runScript();
	
	String		m_ScriptFile;
	bool		m_Initialized;
	PyObject *	m_pPyModule;
	PyObject *	m_pPyGizmoDaemon;
	pthread_mutex_t	m_MutexCallScriptFunc;
	
	// script functions
	static void 		initPyFuncCallbacks();
	static PyObject * 	PythonFunctionRunner(PyObject * self, PyObject * args, int Index);
	
	static int		m_NumFunctions;
	static ScriptFunction * m_Functions [PYTHON_MAX_FUNCTIONS];
	static PyMethodDef * 	m_PyMethods;
	static PyObject * 	(* m_PythonFunctions[PYTHON_MAX_FUNCTIONS])(PyObject *, PyObject *);
	
	static PyObject * 	PythonFunctionHandler000(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler001(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler002(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler003(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler004(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler005(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler006(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler007(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler008(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler009(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler010(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler011(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler012(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler013(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler014(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler015(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler016(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler017(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler018(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler019(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler020(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler021(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler022(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler023(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler024(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler025(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler026(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler027(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler028(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler029(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler030(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler031(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler032(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler033(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler034(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler035(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler036(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler037(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler038(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler039(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler040(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler041(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler042(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler043(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler044(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler045(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler046(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler047(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler048(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler049(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler050(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler051(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler052(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler053(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler054(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler055(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler056(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler057(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler058(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler059(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler060(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler061(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler062(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler063(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler064(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler065(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler066(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler067(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler068(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler069(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler070(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler071(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler072(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler073(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler074(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler075(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler076(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler077(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler078(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler079(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler080(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler081(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler082(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler083(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler084(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler085(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler086(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler087(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler088(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler089(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler090(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler091(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler092(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler093(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler094(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler095(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler096(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler097(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler098(PyObject * self, PyObject * args);
	static PyObject * 	PythonFunctionHandler099(PyObject * self, PyObject * args);
};

//////////////////////////////////////////////////////////////////////////////
// Shared Object Attach Function
///////////////////////////////////////

extern "C" {
	ScriptEngineInterface * getPlugin(LinkList<void> * pArgList) {
		return new ScriptEnginePython();
	}
}

#endif // __ScriptEnginePython_h

