  //////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
//
// GizmoPluginXMMSControl.cpp
//
//	GizmoPluginXMMSControl class
//
///////////////////////////////////////
  ///////////////////////////////////////
  
/*

Copyright (c) 2005, Tim Burrell
All rights reserved.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#include "GizmoPluginXMMSControl.hpp"
#include "libGizmo/Debug.hpp"

////////////////////////////////////////////////////////////////////////////
// Statics / Type Defs
///////////////////////////////////////

////////////////////////////////////////////////////////////////////////////
// Statics / Globals
///////////////////////////////////////

////////////////////////////////////////////////////////////////////////////
// Overrides
///////////////////////////////////////

bool GizmoPluginXMMSControl::onPluginOpened() {
	Debug::outDebug("GizmoPluginXMMSControl :: onPluginOpened");
	
	return initPlugin();
}

////////////////////////////////////////////////////////////////////////////
// Construction
///////////////////////////////////////

GizmoPluginXMMSControl::GizmoPluginXMMSControl(ScriptEngineInterface * pScriptEngine, const char * DevicePath) :
	m_ScriptFuncXMMSPlay(this),
	m_ScriptFuncXMMSGetOutputTime(this),
	m_ScriptFuncXMMSGetPlaylistPosition(this),
	m_ScriptFuncXMMSIsPaused(this),
	m_ScriptFuncXMMSIsPlaying(this),
	m_ScriptFuncXMMSNextTrack(this),
	m_ScriptFuncXMMSPause(this),
	m_ScriptFuncXMMSPreviousTrack(this),
	m_ScriptFuncXMMSSeek(this),
	m_ScriptFuncXMMSSetTrack(this),
	m_ScriptFuncXMMSStop(this)
{
	Debug::outDebug("GizmoPluginXMMSControl :: XMMSControl Output Plugin Loading...");

	m_pScriptEngine = pScriptEngine;
	m_DeviceName = "OutputXMMSControl";
}

GizmoPluginXMMSControl::~GizmoPluginXMMSControl() {
	closePlugin();
	Debug::outDebug("GizmoPluginXMMSControl :: XMMSControl Output Plugin Unloaded.");
}

////////////////////////////////////////////////////////////////////////////
// Class Body
///////////////////////////////////////

void GizmoPluginXMMSControl::closePlugin() {
	Debug::outDebug("GizmoPluginXMMSControl :: XMMSControl Output Plugin Closing...");
	closeXMMSControl();
}

bool GizmoPluginXMMSControl::initPlugin() {
	Debug::out("        - XMMSControl Output Plugin v%s", PACKAGE_VERSION);

	if (!m_pScriptEngine) {
		Debug::outError("GizmoPluginXMMSControl :: Invalid Script Engine!");
		return false;
	}
	
	// register functions
	Debug::outDebug("GizmoPluginXMMSControl :: Registering Functions with Script Engine...");
	m_pScriptEngine->registerScriptFunction(&m_ScriptFuncXMMSGetOutputTime);
	m_pScriptEngine->registerScriptFunction(&m_ScriptFuncXMMSGetPlaylistPosition);
	m_pScriptEngine->registerScriptFunction(&m_ScriptFuncXMMSIsPaused);
	m_pScriptEngine->registerScriptFunction(&m_ScriptFuncXMMSIsPlaying);
	m_pScriptEngine->registerScriptFunction(&m_ScriptFuncXMMSNextTrack);
	m_pScriptEngine->registerScriptFunction(&m_ScriptFuncXMMSPause);
	m_pScriptEngine->registerScriptFunction(&m_ScriptFuncXMMSPlay);
	m_pScriptEngine->registerScriptFunction(&m_ScriptFuncXMMSPreviousTrack);
	m_pScriptEngine->registerScriptFunction(&m_ScriptFuncXMMSSeek);
	m_pScriptEngine->registerScriptFunction(&m_ScriptFuncXMMSSetTrack);
	m_pScriptEngine->registerScriptFunction(&m_ScriptFuncXMMSStop);
	
	Debug::outDebug("GizmoPluginXMMSControl :: XMMSControl Output Plugin Initialized.");
	return true;
}

String GizmoPluginXMMSControl::getDeviceName() {
	return "GizmoPluginXMMSControl";
}

String GizmoPluginXMMSControl::getDeviceNameAlt() {
	return "XMMSControl";
}

int GizmoPluginXMMSControl::xmmsGetPlaylistPosition() {
	return XMMSGetPlaylistPosition();
}

int GizmoPluginXMMSControl::xmmsGetOutputTime() {
	return XMMSGetOutputTime();
}

bool GizmoPluginXMMSControl::xmmsIsPaused() {
	return XMMSIsPaused();
}

bool GizmoPluginXMMSControl::xmmsIsPlaying() {
	return XMMSIsPlaying();
}

bool GizmoPluginXMMSControl::xmmsNextTrack() {
	return XMMSNextTrack();
}

bool GizmoPluginXMMSControl::xmmsPause() {
	return XMMSPause();
}

bool GizmoPluginXMMSControl::xmmsPlay() {
	return XMMSPlay();
}

bool GizmoPluginXMMSControl::xmmsPreviousTrack() {
	return XMMSPreviousTrack();
}

bool GizmoPluginXMMSControl::xmmsSeek(int Offset) {
	return XMMSSeek(Offset);
}

bool GizmoPluginXMMSControl::xmmsSetTrack(int Track) {
	return XMMSSetTrack(Track);
}

bool GizmoPluginXMMSControl::xmmsStop() {
	return XMMSStop();
}

bool GizmoPluginXMMSControl::checkEventsWaiting() {
	return false;
}
